//
// Title:   Configuration.cpp
// Created: Mon Sep 23 13:56:10 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qsize.h>
#include <qstring.h>
#include <qstringlist.h>

#include "../../../Utilities/src/Matrix.h"
#include "../../../DataTypes/src/MyDataTypes.h"
#include "../../../DataTypes/src/Vector3D.h"
#include "Configuration.h"

// ******
// global
// ******

// the single configuration instance, only
// accesible by getInstance() and
// deleteInstance()
static Configuration *configuration = 0;

Configuration::Configuration() {

	// ************************ 
	// init with default values
	// ************************
	viewPortSize.setWidth (400);
	viewPortSize.setHeight (400);

	minimumViewPortSize = viewPortSize;

	viewFrustum.fieldOfView = 30.0f;
	viewFrustum.aspectRatio = 1.0f;
	viewFrustum.nearPlane   = 10.0f;
	viewFrustum.farPlane    = 100000.0f;

	mainShadingEnabled           = true;
	auxiliaryShadingEnabled      = false;
	previewShadingEnabled        = true;
	previewRendererActivated     = false;

	// the available renderer types and the currently used renderers are to be initialized by the RendererManager

	pluginDirectory               = QString ("Plugins");
	rendererPluginDirectory       = QString ("Renderers");
	toolDirectory                 = QString ("Tools");
	fileFormatPluginDirectory     = QString ("FileFormatPlugins");
	brushDirectory				  = QString ("Brushes");
	bitmapDirectory               = QString ("Bitmaps");
	dataDirectory                 = QString ("Models");

	MtrUnity4x4f (homePosition.scaleTranslationMatrix);
	homePosition.scaleTranslationMatrix[14] = -1000.0f;
	MtrUnity4x4f (homePosition.rotationMatrix);

	lightDirection[0] = 0.0f;
	lightDirection[1] = 0.0f;
	lightDirection[2] = -1.0f;

}

Configuration::~Configuration() {}

// *****************
// Singleton Pattern
// *****************

Configuration *Configuration::getInstance() {

	if (configuration == 0) {
		configuration = new Configuration();
	}
	return configuration;

}

void Configuration::destroyInstance() {

	if (configuration != 0) {
		delete configuration;
	}
	configuration = 0;

}

void Configuration::setMainRendererType (const QString newRendererType) {
	mainRendererType = newRendererType;
	emit mainRendererTypeChanged (newRendererType);
}

QString Configuration::getMainRendererType() const {
	return mainRendererType;
}

void Configuration::setAuxiliaryRendererType (const QString newRendererType) {
	auxiliaryRendererType = newRendererType;
	emit auxiliaryRendererTypeChanged (newRendererType);
}

QString Configuration::getAuxiliaryRendererType() const {
	return auxiliaryRendererType;
}

void Configuration::setPreviewRendererType (const QString newRendererType) {
	previewRendererType = newRendererType;
	emit previewRendererTypeChanged (newRendererType);
}

QString Configuration::getPreviewRendererType() const {
	return previewRendererType;
}

void Configuration::setAvailableRendererTypes (const QStringList newAvailableRendererTypes) {
	availableRendererTypes = newAvailableRendererTypes;
	emit availableRendererTypesChanged (newAvailableRendererTypes);
}

QStringList Configuration::getAvailableRendererTypes() const {
	return availableRendererTypes;
}

void Configuration::setPreviewRendererEnabled (const bool enable) {
	previewRendererActivated = enable;
	emit previewRendererEnabled (enable);
}

bool Configuration::isPreviewRendererEnabled() const {
	return previewRendererActivated;
}

void Configuration::setViewPortSize (const QSize newViewPortSize) {
	viewPortSize = newViewPortSize;
	emit viewPortSizeChanged (newViewPortSize);
}

QSize Configuration::getViewPortSize() const {
	return viewPortSize;
}

QSize Configuration::getMinimumViewPortSize() const {
	return minimumViewPortSize;
}

void Configuration::setViewFrustum (const MyDataTypes::ViewFrustum newViewFrustum) {
	viewFrustum = newViewFrustum;
	emit viewFrustumChanged (newViewFrustum);
}

MyDataTypes::ViewFrustum Configuration::getViewFrustum() const {
	return viewFrustum;
}

void Configuration::setMainShadingEnabled (const bool enable) {
	mainShadingEnabled = enable;
	emit mainShadingChanged (enable);
}

bool Configuration::isMainShadingEnabled() const {
	return mainShadingEnabled;
}

void Configuration::setAuxiliaryShadingEnabled (const bool enable) {
	auxiliaryShadingEnabled = enable;
	emit auxiliaryShadingChanged (enable);
}

bool Configuration::isAuxiliaryShadingEnabled() const {
	return auxiliaryShadingEnabled;
}

void Configuration::setPreviewShadingEnabled (const bool enable) {
	previewShadingEnabled = enable;
	emit previewShadingChanged (enable);
}

bool Configuration::isPreviewShadingEnabled() const {
	return previewShadingEnabled;
}

void Configuration::setPluginDirectory (const QString newPluginDirectory) {
	pluginDirectory = newPluginDirectory;
	emit pluginDirectoryChanged (newPluginDirectory);
}

QString Configuration::getPluginDirectory() const {
	return pluginDirectory;
}

void Configuration::setRendererPluginDirectory (const QString newRendererPluginDirectory) {
	rendererPluginDirectory = newRendererPluginDirectory;
	emit rendererPluginDirectoryChanged (newRendererPluginDirectory);
}

QString Configuration::getRendererPluginDirectory() const {
	return rendererPluginDirectory;
}

void Configuration::setToolDirectory (const QString newToolDirectory) {
	toolDirectory = newToolDirectory;
	emit toolDirectoryChanged (newToolDirectory);
}

QString Configuration::getToolDirectory() const {
	return toolDirectory;
}

void Configuration::setBrushDirectory (const QString newBrushDirectory) {
	brushDirectory = newBrushDirectory;
	emit brushDirectoryChanged (newBrushDirectory);
}

QString Configuration::getBrushDirectory() const {
	return brushDirectory;
}

void Configuration::setFileFormatPluginDirectory (const QString newFileFormatPluginDirectory) {
	fileFormatPluginDirectory = newFileFormatPluginDirectory;
	emit fileFormatPluginDirectoryChanged (newFileFormatPluginDirectory);
}

QString Configuration::getFileFormatPluginDirectory() const {
	return fileFormatPluginDirectory;
}

void Configuration::setBitmapDirectory (const QString newBitmapDirectory) {
	bitmapDirectory = newBitmapDirectory;
	emit bitmapDirectoryChanged (newBitmapDirectory);
}

QString Configuration::getBitmapDirectory() const {
	return bitmapDirectory;
}

void Configuration::setDataDirectory (const QString newDataDirectory) {
	dataDirectory = newDataDirectory;
	emit dataDirectoryChanged (newDataDirectory);
}

QString Configuration::getDataDirectory() const {
	return dataDirectory;
}

void Configuration::setHomePosition (const MyDataTypes::CameraPosition newHomePosition) {
	
	homePosition = newHomePosition;
	emit homePositionChanged (newHomePosition);

}

MyDataTypes::CameraPosition Configuration::getHomePosition() const {
	return homePosition;
}

void Configuration::setLightDirection (const Vector3D newLightDirection) {
	lightDirection = newLightDirection;
	lightDirection.normalize();
	emit lightDirectionChanged (newLightDirection);
}

Vector3D Configuration::getLightDirection() const {
	return lightDirection;
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
