//
// Title:   ColorPickerTool.h
// Created: Mon Sep 23 13:50:01 2002
// Authors: Oliver Knoll
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __COLORPICKERTOOL_H_
#define __COLORPICKERTOOL_H_

#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qcursor.h>
#include <qevent.h>
#include <qwidget.h>
#include <qobject.h>
#include <qpixmap.h>

#include "../../../Interfaces/src/ToolInterface.h"
#include "../../../CoreResourceTools/ColorChooserTool/src/ColorChooserTool.h"

/**
 * This class implements a tool with which allows the user to pick the
 * foreground and background color in the rendered image. The <code>ColorChooserTool</code>
 * is updated accordingly.
 *
 * @see ColorChooserTool
 * @author Oliver Knoll
 * @version 1.2
 */
class ColorPickerTool : public ToolInterface {

	Q_OBJECT

public:
	
	TOOL_API ColorPickerTool();
	TOOL_API virtual ~ColorPickerTool();

	/**
	 * Makes the <code>ColorChooserTool</code> known to this <code>ColorPickerTool</code>
	 * so it can be updated according to the picked color.
	 *
	 * @param colorChooserTool
	 *        a pointer to the <code>ColorChooserTool</code> which gets updated each time
	 *        a new color has been picked; may be 0, in which case the <code>ColorChooserTool</code>
	 *        is not updated
	 */ 
	TOOL_API void setupColorChooserTool (ColorChooserTool *colorChooserTool);

	/**
	 * Picks a foreground (FG) color at mouse (screen) coordinates <code>mouseX</code>
	 * and <code>mouseY</code>.
	 *
	 * @param mouseX
	 *        the x-coordinate of the mouse at which the color is to be picked
	 * @param mouseY
	 *        the y-coordinate of the mouse at which the color is to be picked
	 * @return the picked QRgb color
	 */
	TOOL_API QRgb pickFGColor (const int mouseX, const int mouseY);

	/**
	 * Picks a background (bg) color at mouse (screen) coordinates <code>mouseX</code>
	 * and <code>mouseY</code>.
	 *
	 * @param mouseX
	 *        the x-coordinate of the mouse at which the color is to be picked
	 * @param mouseY
	 *        the y-coordinate of the mouse at which the color is to be picked
	 * @return the picked QRgb color
	 */
	TOOL_API QRgb pickBGColor (const int mouseX, const int mouseY);
	
	// ***********************
	// interface ToolInterface
	// ***********************

	TOOL_API QToolButton *addButtonToToolBar (QToolBar *toolBar);
	TOOL_API QToolButton *getToolButton();
	TOOL_API QCursor *getToolCursor() const;
	TOOL_API QWidget *getToolConfigurationWidget() const;
	TOOL_API bool isAvailable() const;
	
	TOOL_API void handleMousePressEvent   (QMouseEvent *me);
	TOOL_API void handleMouseReleaseEvent (QMouseEvent *me);
	TOOL_API void handleMouseMoveEvent    (QMouseEvent *me);
	TOOL_API void handleKeyPressEvent     (QKeyEvent   *ke);
	TOOL_API void handleKeyReleaseEvent   (QKeyEvent   *ke);

	TOOL_API void selectTool();
	TOOL_API void deselectTool();

public slots:

	void handleRendererWidgetRepainted();

private:

	QCursor          *toolCursor;
	QToolButton      *toolButton;
	ColorChooserTool *colorChooserTool;   // a reference to the color chooser tool
	                                      // so we can update the current chosen color
	                                      // according to the picked color
	
	
	// picks a color at mouse (screen) coordinates mouseX and mouseY
	QRgb pickColor (const int mouseX, const int mouseY);

	bool checkToolAvailability() const;
	
};

#endif  // __COLORPICKERTOOL_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
