//
// Title:   SelectionTool.h
// Created: Mon Sep 23 13:49:16 2002
// Authors: Oliver Knoll, Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SELECTIONTOOL_H_
#define __SELECTIONTOOL_H_

#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qcursor.h>
#include <qevent.h>
#include <qwidget.h>
#include <qobject.h>
#include <qpopupmenu.h>
#include <qpointarray.h>
#include <qlist.h>

#include "../../../Interfaces/src/ToolInterface.h"
#include "../../../Interfaces/src/SurfelInterface.h"
#include "../../../OrthoMapper/src/OrthoMapper.h"
#include "../UI/SelectionConfigurationWidget.h"

/**
 * This class implements a selection tool. The selection tool provides several tools such
 * as rectangle selection, polygon selection, ellipse selection, lasso selection and brush painting selection.
 * Currently the system supports three different selections that can be activated by the user using the tool settings
 * dialog. The selections are defined by the flags <code>SELECTED1</code>, <code>SELECTED2</code> and <code>SELECTED3</code>.
 * Only one selection is active at a time and in the documentation we use the symbol <code>SELECTED*</code> to refer
 * to the currently active selection.
 *
 * @author Oliver Knoll, Mark Pauly
 * @version 1.2
 */

class SelectionTool : public ToolInterface {

	Q_OBJECT

public:
	
	TOOL_API SelectionTool();
	TOOL_API virtual ~SelectionTool();


	/**
	 * Puts all visible <code>SurfelInterface</code>s into the currently active selection, that is, the <code>SELECTED*</code> and
	 * <code>EMPHASISE</code> flag are set to <code>true</code>. The image is re-rendered if
	 * <code>visualizeSelection</code> is set to <code>true</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>selectionChanged</code></li>
	 * </ul>
	 * @param visualizeSelection
	 *        set to <code>true</code> if the image should be re-rendered; <code>false</code> else
	 * @see #selectionChanged
	 */
	TOOL_API void selectAllVisible (const bool visualizeSelection);

	/**
	 * Selects all <code>SurfelInterface</code>s in the <em>active</em> <code>Object</code>,
	 * that is, the <code>SELECTED*</code> and
	 * <code>EMPHASISE</code> flag are set to <code>true</code>. The image is re-rendered if
	 * <code>visualizeSelection</code> is set to <code>true</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>selectionChanged</code></li>
	 * </ul>
	 * @param visualizeSelection
	 *        set to <code>true</code> if the image should be re-rendered; <code>false</code> else
	 * @see Scene#getActiveObject
	 * @see #selectionChanged
	 */
	TOOL_API void selectAllActiveObject (const bool visualizeSelection);

	/**
	 * Returns the selected <code>SurfelInterface</code>s in a <code>QVector</code>. If no surfels are
	 * selected, the returned <code>QVector</code> doesn't contain any pointers to surfels.
	 * Note that in the later case <code>hasSelection</code> returns <code>false</code>.
	 * 
	 * @param selectionFlag
	 *        the flag specifying the selection
	 * @return a pointer to a <code>QVector</code> which contains pointers to the selected 
	 *         <code>SurfelInterface</code>s
	 * @see #hasSelection
	 */
	TOOL_API QVector<SurfelInterface> *getSelection (SurfelInterface::Flag selectionFlag = SurfelInterface::SELECTED1);

	/**
	 * Returns the <code>SurfelInterface</code>s in a <code>QVector</code> that are <em>not</em> selected. If all surfels are
	 * selected, the returned <code>QVector</code> doesn't contain any pointers to surfels.
	 * 
	 * @param allNotSelected
	 *         the <code>QVector</code> which contains pointers to the not selected <code>SurfelInterface</code>s
	 * @param selectionFlag
	 *        the flag specifying the selection
	 */
	TOOL_API void getAllNotSelected (QVector<SurfelInterface> &allNotSelected, SurfelInterface::Flag selectionFlag);



	/**
	 * Returns the number of surfels in the selection
	 * 
	 * @param selectionFlag
	 *        the flag specifying the selection
	 */
	TOOL_API int getSelectionSize (SurfelInterface::Flag selectionFlag = SurfelInterface::SELECTED1);


	/**
	 * Deselects all selected <code>SurfelInterface</code>s in the active selection, that is, the <code>SELECTED*</code> and <code>EMPHASISE</code>
	 * flag are set to <code>false</code>.
	 * The image is re-rendered if <code>visualizeSelection</code> is set to <code>true</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>selectionChanged</code></li>
	 * </ul>
	 *
	 * @param visualizeSelection
	 *        set to <code>true</code> if the image should be re-rendered; <code>false</code> else
	 * @see #selectionChanged
	 */
	TOOL_API void clearSelection (const bool visualizeSelection, const bool emitSignal = true);


	/**
	 * Inverts the active selection, that is, the <code>SELECTED*</code> flag is inverted.
	 * The image is re-rendered if <code>visualizeSelection</code> is set to <code>true</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>selectionChanged</code></li>
	 * </ul>
	 *
	 * @param visualizeSelection
	 *        set to <code>true</code> if the image should be re-rendered; <code>false</code> else
	 * @see #selectionChanged
	 */
	TOOL_API void invertSelection (const bool visualizeSelection);


	/**
	 * Returns <code>true</code>, if the active selection is not empty. The selected surfels can be received with
	 * <code>getSelection</code>.
	 *
	 * @return <code>true</code>, if there are selected surfels
	 * @see #getSelection
	 */
	TOOL_API bool hasSelection();

	/**
	 * Sets the active selection
	 * 
	 * @param activeSelection
	 *        index into the list of selections
	 *
	 */
	TOOL_API void setActiveSelection (int activeSelection);

	/**
	 * Gets the index of the active selection
	 * 
	 * @return  the index of the active selection
	 *
	 */
	TOOL_API int getActiveSelectionIndex () const;
	

	TOOL_API void updateSelectionCounters ();


	/**
	 * Emits the signal <code>selectionChanged</code> for all selections.
	 *
	 * @see #selectionChanged
	 */
	TOOL_API void emitAllSelectionsChanged();

	// ***********************
	// interface ToolInterface
	// ***********************

	TOOL_API QToolButton *addButtonToToolBar (QToolBar *toolBar);
	TOOL_API QToolButton *getToolButton();
	TOOL_API QCursor *getToolCursor() const;
	TOOL_API QWidget *getToolConfigurationWidget() const;
	TOOL_API bool isAvailable() const;
	
	TOOL_API void handleMousePressEvent   (QMouseEvent *me);
	TOOL_API void handleMouseReleaseEvent (QMouseEvent *me);
	TOOL_API void handleMouseMoveEvent    (QMouseEvent *me);
	TOOL_API void handleKeyPressEvent     (QKeyEvent   *ke);
	TOOL_API void handleKeyReleaseEvent   (QKeyEvent   *ke);

	TOOL_API void selectTool();
	TOOL_API void deselectTool();

signals:

	/**
	 * Emitted whenever the selection has changed. The <code>selection</code> contains the selected
	 * <code>SurfelInterface</code>s.
	 *
	 * @param selectionFlag
	 *        flag of the modified selection
	 * <code>SurfelInterface</code>s
	 */
	void selectionChanged (SurfelInterface::Flag selectionFlag);

public slots:

	void handleRendererWidgetRepainted();
	
private:

	SelectionConfigurationWidget *configurationWidget;
	QCursor                *toolCursor;
	QIconSet               ellipseIconSet,
                           lassoIconSet,
					       polygonIconSet,
                           rectangleIconSet,
						   brushIconSet;
	QToolButton            *toolButton;
	QPopupMenu             selectionMenu;
	int                    ellipseID,
	                       lassoID,
					       polygonID,
	                       rectangleID,
						   brushID,
	                       currentTool;
	QRect                  currentEllipseRect,
		                   currentRectangleRect;
	QPointArray            currentPolyLine;
	
	uint                   nofPolyPoints;   // number of points in the polyline
	bool                   isPolygonLineBeingConstructed;
	QPoint                 oldMousePosition,
		                   startMousePosition;
	QPoint                 lastMouseDragPosition;
	Vector2D			   screenPosition;

	QVector<SurfelInterface> *selectionLists;
	bool                   *selectionsValid;

	int                    *selectionsNumSurfels;


	bool                   currentPositionValid;
	OrthoMapper            *orthoMapper;

	void initiateSelectOperation (bool select);
	SurfelInterface::Flag  getCurrentSelectionFlag();
	int getSelectionIndex (SurfelInterface::Flag selectionFlag);
	SurfelInterface::Flag getSelectionFlag (int selectionIndex);

	bool checkToolAvailability() const;

private slots:

	void handleToolSelected (int id);
	void handleActiveObjectChanged();
	void handleObjectModified (bool samplingPatternAltered);

};

#endif  // __SELECTION_TOOL_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
