//
// Title:   NeighbourHood.cpp
// Created: Mon Sep 23 13:47:35 2002
// Authors: Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "../../../DataTypes/src/Vector3D.h"
#include "../../../Utilities/ANN/ANN.h"

#include "NeighbourHood.h"

NeighbourHood::NeighbourHood (const Vector3D *positions, const uint nofPositions) {

	this->positions    = positions;
	this->nofPositions = nofPositions;
	nofNeighbours      = 1;
	
	kdTree       = 0;
	positionsANN = 0;
	this->rebuildKDTree();

	indices          = new ANNidx[nofNeighbours];
	squaredDistances = new ANNdist[nofNeighbours];

}

NeighbourHood::~NeighbourHood() {

	if (kdTree != 0) {
		delete kdTree;
	}
	
	if (positionsANN != 0) {
		annDeallocPts (positionsANN);
	}

	delete[] indices;
	delete[] squaredDistances;

}

void NeighbourHood::setPositions (const Vector3D *newPositions, const uint newNofPositions) {

	positions       = newPositions;
	nofPositions    = newNofPositions;
	this->rebuildKDTree();

}

const Vector3D *NeighbourHood::getPositions() const {
	return positions;
}

uint NeighbourHood::getNofPositions() const {
	return nofPositions;
}

void NeighbourHood::setNofNeighbours (const uint newNofNeighbours) {

	// remove the old data
	delete[] indices;
	delete[] squaredDistances;

	// allocate new data
	nofNeighbours    = newNofNeighbours;
	indices          = new ANNidx[nofNeighbours];
	squaredDistances = new ANNdist[nofNeighbours];

	neighboursUpdated = false;

}

void NeighbourHood::setSourcePoint (const Vector3D newSourcePoint) {
	sourcePoint       = newSourcePoint;
	neighboursUpdated = false;
}

Vector3D NeighbourHood::getSourcePoint() const {
	return sourcePoint;
}

uint NeighbourHood::getNofNeighbours() const {
	return nofNeighbours;
}

uint NeighbourHood::getNeighbourPositionIndex (const uint neighbourIndex) {

	if (neighboursUpdated == false) {
		// we have to compute the neighbours first
		kdTree->annkSearch (sourcePoint.source(), nofNeighbours, indices, squaredDistances, 0);
		neighboursUpdated = true;
	}

	return indices[neighbourIndex];
	
}


Vector3D NeighbourHood::getNeighbourPosition (const uint neighbourIndex) {

	if (neighboursUpdated == false) {
		// we have to compute the neighbours first
		kdTree->annkSearch (sourcePoint.source(), nofNeighbours, indices, squaredDistances, 0);
		neighboursUpdated = true;
	}
	return positions[getNeighbourPositionIndex (neighbourIndex)];
}


float NeighbourHood::getSquaredDistance (const uint neighbourIndex) {

	if (neighboursUpdated == false) {
		// we have to compute the neighbours first
		kdTree->annkSearch (sourcePoint.source(), nofNeighbours, indices, squaredDistances, 0);
		neighboursUpdated = true;
	}
	return  squaredDistances[neighbourIndex];
}



// ***************
// private methods
// ***************

void NeighbourHood::rebuildKDTree() {

	Vector3D      position;
	uint          i;

	// delete the old KDTree and its point array, if necessary
	if (kdTree != 0) {
		delete kdTree;
	}

	if (positionsANN != 0) {
		// remove ANN points
		annDeallocPts (positionsANN);
	}

	// build search structure

	// allocate conversion structure
    positionsANN = annAllocPts (nofPositions, 3);

	for (i = 0; i < nofPositions; i++) {

		position = positions[i];
		positionsANN[i][0] = position[0];
		positionsANN[i][1] = position[1];
		positionsANN[i][2] = position[2];
	
	}

	kdTree = new ANNkd_tree (positionsANN,	nofPositions, 3);
	neighboursUpdated = false;	

}



// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
