//
// Title:   SurfelCollection.cpp
// Created: Mon Sep 23 13:47:19 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qlist.h>
#include <qdatastream.h>

#include "../../../Utilities/UserMessages/src/StatusBar.h"
#include "../../../DataTypes/src/BaseSurfel.h"
#include "../../../DataTypes/src/Surfel1.h"
#include "../../../DataTypes/src/ElSurfel.h"
#include "../../../DataTypes/src/Vector3D.h"
#include "../../MarkerManager/src/MarkerManager.h"
#include "SurfelCollection.h"

// **************
// static members
// **************

const MyDataTypes::TextureCoordinate SurfelCollection::defaultTextureCoordinate = {0.0f, 0.0f};

// **************
// public methods
// **************

SurfelCollection::SurfelCollection () {	
	propertyDescriptor = BaseSurfel::PROPERTY_DESCRIPTOR;
	surfels.setAutoDeleteEnabled (true);
}

SurfelCollection::~SurfelCollection() {}

void SurfelCollection::setPropertyDescriptor (const SurfelInterface::PropertyDescriptor newPropertyDescriptor) {

	SurfelInterface::PropertyDescriptor matchingDescriptor = SurfelInterface::NO_PROPERTIES;
	size_t                              surfelSize,
		                                minimumSize;
	uint                                i,
		                                nofSurfels;
	SurfelInterface                     *oldSurfel,
		                                *newSurfel;
	Marker3D                            *marker3D;
	bool                                matchFound;

	// iterate through all available (hard-coded) property-descriptors, check for a match
	// and take the smallest (in terms of size in bytes) surfel type which matches the requested properties

	// finding a match:
	// "everywhere where's a 1 in the newPropertyDescriptor, there must be a 1 in the
	//  available property descriptor"
	// 1. invert the available PropertyDescriptors
	// 2. bitwise AND them with the newPropertyDescriptor
	// 3. result must be 0 => we have a match
	// [thanks to Christian Gloor ;)]

	minimumSize = 1000000;
	matchFound = false;

	// BaseSurfel:
	if (((~BaseSurfel::PROPERTY_DESCRIPTOR) & newPropertyDescriptor) == 0) {
		minimumSize = sizeof (BaseSurfel);
		matchingDescriptor = BaseSurfel::PROPERTY_DESCRIPTOR;
		matchFound = true;
	}

	// Surfel1:
	if (((~Surfel1::PROPERTY_DESCRIPTOR) & newPropertyDescriptor) == 0) {

		surfelSize = sizeof (Surfel1);
		if (surfelSize < minimumSize) {
			matchingDescriptor = Surfel1::PROPERTY_DESCRIPTOR;
			minimumSize = surfelSize;
			matchFound = true;
		}
	}

	// ElSurfel:
	if (((~ElSurfel::PROPERTY_DESCRIPTOR) & newPropertyDescriptor) == 0) {
		
		surfelSize = sizeof (ElSurfel);
		if (surfelSize < minimumSize) {
			matchingDescriptor = ElSurfel::PROPERTY_DESCRIPTOR;
			minimumSize = surfelSize;
			matchFound = true;
		}		
	}
	
	if (matchFound) {
		
		// check if property descriptor has really changed, i.e. if we need to convert this SurfelCollection
		if (matchingDescriptor != propertyDescriptor) {
			
			StatusBar::getInstance()->showMessage ("Converting surfel properties...", -1);
			
			propertyDescriptor = matchingDescriptor;
			nofSurfels = surfels.count();
			// TODO: create new types here
			if (propertyDescriptor == BaseSurfel::PROPERTY_DESCRIPTOR) {
				
				for (i = 0; i < nofSurfels; i++) {
					
					// the new current item is now the first one
					oldSurfel = surfels.first();
					newSurfel = BaseSurfel::create (oldSurfel->getPosition(), oldSurfel->getNormal(),
						oldSurfel->getRadius(), oldSurfel->getDiffuseColor(),
						oldSurfel->getSpecularColor(), 
						oldSurfel->getFlags());
					
					// check if there is a marker associated with the old surfel. if so, update the marker
					// to reference the new surfel.
					if((marker3D = MarkerManager::getInstance()->getMarker3D(oldSurfel)) != NULL) {
						marker3D->setSurfel(newSurfel);
					}
					
					// delete the old (current) surfel and insert the new surfel;
					surfels.remove();
					surfels.append (newSurfel);
					
				}
				
			}
			else if (propertyDescriptor == Surfel1::PROPERTY_DESCRIPTOR) {
				
				for (i = 0; i < nofSurfels; i++) {
					
					// the new current item is now the first one
					oldSurfel = surfels.first();
					newSurfel = Surfel1::create (oldSurfel->getPosition(),
						oldSurfel->getNormal(),
						oldSurfel->getRadius(),
						oldSurfel->getDiffuseColor(),
						oldSurfel->getSpecularColor(),
						oldSurfel->getFlags(),
						oldSurfel->getAmbientCoefficient(),
						oldSurfel->getDiffuseCoefficient(),
						oldSurfel->getSpecularCoefficient(),
						oldSurfel->getShininess());
					
					// check if there is a marker associated with the old surfel. if so, update the marker
					// to reference the new surfel.
					if((marker3D = MarkerManager::getInstance()->getMarker3D(oldSurfel)) != NULL) {
						marker3D->setSurfel(newSurfel);
					}
					
					// delete the old (current) surfel and insert the new surfel;
					surfels.remove();
					surfels.append (newSurfel);
				}
			}
			
			else if (propertyDescriptor == ElSurfel::PROPERTY_DESCRIPTOR) {
				
				for (i = 0; i < nofSurfels; i++) {
					
					// TODO:: axis have to be computed!

					// the new current item is now the first one
					oldSurfel = surfels.first();
					newSurfel = ElSurfel::create (oldSurfel->getPosition(),
						oldSurfel->getNormal(),
						oldSurfel->getAxisOne(),
						oldSurfel->getAxisTwo(),
						oldSurfel->getDiffuseColor(),
						oldSurfel->getFlags());
					
					// check if there is a marker associated with the old surfel. if so, update the marker
					// to reference the new surfel.
					if((marker3D = MarkerManager::getInstance()->getMarker3D(oldSurfel)) != NULL) {
						marker3D->setSurfel(newSurfel);
					}
					
					// delete the old (current) surfel and insert the new surfel;
					surfels.remove();
					surfels.append (newSurfel);
				}
			}
			
			
			emit surfelTypeChanged (propertyDescriptor);
			StatusBar::getInstance()->showMessage ("Surfel properties converted.");
			
		}
	}
}

SurfelInterface::PropertyDescriptor SurfelCollection::getPropertyDescriptor() const {
	return propertyDescriptor;
}

uint SurfelCollection::getNofSurfels() const {
	return surfels.count();
}

SurfelInterface *SurfelCollection::getFirstSurfel() {
	return surfels.first();
}

SurfelInterface *SurfelCollection::getCurrentSurfel() const {
	return surfels.current();
}

SurfelInterface *SurfelCollection::getNextSurfel() {
	return surfels.next();
}

bool SurfelCollection::contains (const SurfelInterface *surfel) {
	return (surfels.find (surfel) != -1);
}

SurfelInterface *SurfelCollection::addSurfel (const Vector3D position, const Vector3D normal, 
											  const Vector3D axisOne, const Vector3D axisTwo, const float radius,
								              const QRgb diffuseColor, const QRgb specularColor, const SurfelInterface::Flags flags,
											  const float ambientCoefficient, const float diffuseCoefficient,
											  const float specularCoefficient, const float shininess,
											  const MyDataTypes::TextureCoordinate textureCoordinate,
								              const bool emitSignal) {

	SurfelInterface *surfel = NULL;

	// TODO: create new types here
	if (propertyDescriptor == BaseSurfel::PROPERTY_DESCRIPTOR) {
		surfel = BaseSurfel::create (position, normal, radius, diffuseColor, specularColor, flags);
	}
	else if (propertyDescriptor == Surfel1::PROPERTY_DESCRIPTOR) {
		surfel = Surfel1::create (position, normal, radius, diffuseColor, specularColor, flags,
			                      ambientCoefficient, diffuseCoefficient, specularCoefficient, shininess, textureCoordinate);
	}
	else if (propertyDescriptor == ElSurfel::PROPERTY_DESCRIPTOR) {
		surfel = ElSurfel::create (position, normal, axisOne, axisTwo, diffuseColor, flags);
	}

	surfels.append (surfel);
	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}
	return surfel;

}


SurfelInterface *SurfelCollection::addSurfel (const bool emitSignal) {

	SurfelInterface *surfel = NULL;

	// TODO: create new types here
	if (propertyDescriptor == BaseSurfel::PROPERTY_DESCRIPTOR) {
		surfel = BaseSurfel::create();
	}
	else if (propertyDescriptor == Surfel1::PROPERTY_DESCRIPTOR) {
		surfel = Surfel1::create();			
	}
	else if (propertyDescriptor == ElSurfel::PROPERTY_DESCRIPTOR) {
		surfel = ElSurfel::create();			
	}

	surfels.append (surfel);
	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}
	return surfel;

}

SurfelInterface *SurfelCollection::addSurfel (const SurfelInterface *surfel, const bool emitSignal) {

	SurfelInterface* newSurfel = NULL;

	// create surfel of correct type
	if (propertyDescriptor == BaseSurfel::PROPERTY_DESCRIPTOR) {
		newSurfel = BaseSurfel::create();
	}
	else if (propertyDescriptor == Surfel1::PROPERTY_DESCRIPTOR) {
		newSurfel = Surfel1::create();			
	}
	else if (propertyDescriptor == ElSurfel::PROPERTY_DESCRIPTOR) {
		newSurfel = ElSurfel::create();			
	}

	// copy attributes from given surfel
	newSurfel->copy(surfel);
	
	surfels.append (newSurfel);
	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

	return newSurfel;

}

void SurfelCollection::addSurfels (const QVector<SurfelInterface> *newSurfels, const bool emitSignal) {


	unsigned int i;
	for (i = 0 ; i < newSurfels->count(); i++) {
		this->addSurfel (newSurfels->at(i), false);      
    }

	if (emitSignal) {
		emit surfelCollectionChanged();
	}

}

void SurfelCollection::merge (const SurfelCollection *addSurfels, const bool emitSignal) {

	surfels.append (&addSurfels->surfels);

	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

}

void SurfelCollection::removeCurrentSurfel () {
	surfels.remove();
}

bool SurfelCollection::removeSurfel (SurfelInterface *surfel, const bool emitSignal) {

	bool result;

	result = surfels.remove (surfel);
	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

	return result;

}

void SurfelCollection::removeSurfels (const QList<SurfelInterface> *removeSurfels, const bool emitSignal) {

	QListIterator<SurfelInterface> it(*removeSurfels);

	for ( ; it.current() != 0; ++it) {
		surfels.remove (it.current());     
    }

	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

}


void SurfelCollection::removeSurfels (QArray<SurfelInterface*> removeSurfels, const int numSurfels, const bool emitSignal) {

	int i;

	for (i = 0; i < numSurfels; i++) {
		this->surfels.remove (removeSurfels[i]);
	}

	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

}

void SurfelCollection::copy (SurfelCollection *sourceCollection, const bool emitSignal) {

	SurfelInterface  *surfel;
	SurfelList *sourceSurfels;

	sourceSurfels = &(sourceCollection->surfels);
	surfels.clear();
	
	// copy the surfels
	for (surfel = sourceSurfels->first(); surfel != 0; surfel = sourceSurfels->next()) {
		surfels.append (surfel->copy());
	}

	// copy the property descriptor
	propertyDescriptor = sourceCollection->propertyDescriptor;

	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

}

SurfelCollection *SurfelCollection::copy() {

	SurfelCollection *deepCopy;
	SurfelInterface *surfel;
	SurfelList *destinationSurfels;

	deepCopy = new SurfelCollection();
	destinationSurfels = &(deepCopy->surfels); 
	
	// copy the surfels
	for (surfel = surfels.first(); surfel != 0; surfel = surfels.next()) {
		destinationSurfels->append (surfel->copy());
	}
	
	// copy the property descriptor
	deepCopy->propertyDescriptor = propertyDescriptor;

	return deepCopy;

}

void SurfelCollection::clear (const bool emitSignal) {

	surfels.clear();
	if (emitSignal == true) {
		emit surfelCollectionChanged();
	}

}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
