//
// Title:   SurfelCollection.h
// Created: Mon Sep 23 13:47:12 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SURFELCOLLECTION_H_
#define __SURFELCOLLECTION_H_

#include <qobject.h>
#include <qlist.h>
#include <qdatastream.h>

#include "../../../DataTypes/src/MyDataTypes.h"
#include "../../../DataTypes/src/Vector3D.h"
#include "../../../DataTypes/src/Surfel1.h"
#include "../../../DataTypes/src/ElSurfel.h"
#include "../../Interfaces/src/SurfelInterface.h"
#include "../../src/CoreDLL.h"
#include "SurfelList.h"

/**
 * The <code>SurfelCollection</code> contains all the <code>BaseSurfel</code>s (or their subclasses) which make up
 * an <code>Object</code>.
 *
 * <b>Note:</b> The surfel collection is value-based. I.e., surfel objects only exist in surfel collections, in 
 * other words, a pointer to a surfel that is not contained in any surfel collection is meaningless. Pointers
 * to surfels should rather be treated as "references" or "indices". Removing a surfel from a surfel collection
 * always means discarding its attribute data and de-allocating its memory.
 *
 * @author Oliver Knoll, Mark Pauly, Matthias Zwicker
 * @version 1.2
 */
class SurfelCollection : public QObject {

	Q_OBJECT

public:

	/**
	 * The defaul parameter for texture UV coordiantes with value (0.0, 0.0).
	 */
	CORE_API static const MyDataTypes::TextureCoordinate defaultTextureCoordinate;

	/**
	 * Creates this <code>SurfelCollection</code>.
	 */
	CORE_API SurfelCollection ();

	/**
	 * Destroys this <code>SurfelCollection</code>. All surfels are deleted, too. All references to these
	 * surfels become invalid!
	 */
	CORE_API virtual ~SurfelCollection();

	/** 
	 * Converts all the types of the surfels which implement the <code>SurfelInterface</code>s so they
	 * support the properties requested in <code>newPropertyDescriptor. If less <code>Properties</code>
	 * are requested than there are already supported then the superfluous <code>Properties</code> are simply discarded.
	 * If more <code>Properties</code> are requested the additional <code>Properties</code> are set to
	 * default values.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelTypeChanged</code></li>
	 * </ul>
	 *
	 * @param newPropertyDescriptor
	 *        the <code>PropertyDescriptor</code> which consists of up to 32 OR'ed <code>Properties</code>
	 * @see SurfelInterface#Property
	 * @see #surfelTypeChanged
	 */
	CORE_API void setPropertyDescriptor (const SurfelInterface::PropertyDescriptor newPropertyDescriptor);

	/**
	 * Returns the <code>PropertyDescriptor</code>, which specifies which properties the <code>SurfelInterface</code>s
	 * in this <code> SurfelCollection</code> support.
	 *
	 * @return the <code>PropertyDescriptor</code> which consists of up to 32 OR'ed <code>Properties</code>
	 * @see #Property
	 */
	CORE_API SurfelInterface::PropertyDescriptor getPropertyDescriptor() const;

	/**
	 * Returns the number of <code>SurfelInterface</code>s in this <code>SurfelCollection</code>.
	 *
	 * @return the number of <code>SurfelInterface</code>s in this <code>SurfelCollection</code>
	 */
	CORE_API uint getNofSurfels() const;

	// ***********
	// Data access
	// ***********

	/**
	 * Returns the first <code>SurfelInterface</code> in this <code>SurfelCollection</code> and
	 * sets the internal iterator to it.
	 *
	 * @see #getNextSurfel
	 * @see #getCurrentSurfel
	 */
	CORE_API SurfelInterface *getFirstSurfel();

	/**
	 * Returns the current <code>SurfelInterface</code> in this <code>SurfelCollection</code> according
	 * to the internal iterator.
	 *
	 * @return a pointer to the current <code>SurfelInterface</code>; 0 is returned if there are no
	 *         <code>)SurfelInterface</code>s to be returned
	 * @see #getFirstSurfel
	 * @see #getNextSurfel
	 */
	CORE_API SurfelInterface *getCurrentSurfel() const;

	/**
	 * Returns the next <code>SurfelInterface</code> in this <code>SurfelCollection</code> and
	 * sets the internal iterator to it.
	 *
	 * @return a pointer to the next <code>SurfelInterface</code>; 0 is returned if there are no more
	 *         <code>)SurfelInterface</code>s to be returned
	 * @see #getFirstSurfel
	 * @see #getCurrentSurfel
	 */
	CORE_API SurfelInterface *getNextSurfel();	

	// ********************
	// Surfel manipulations
	// ********************

	/**
	 * Check if the surfel collection contains a given surfel.
	 */
	CORE_API bool contains (const SurfelInterface *surfel);

	/**
	 * Adds a new <code>SurfelInterface</code> with the given properties (all other available properties are
	 * set to default values) to this <code>SurfelCollection</code> and returns a pointer to it.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param position
	 *        a <code>Vector3D</code> specifying the position of the surfel
	 * @param normal
	 *        a <code>Vector3D</code> specifying the normal of the surfel
	 * @param axisOne
	 *        a <code>Vector3D</code> specifying the tangent axisOne of the surfel
	 * @param axisTwo
	 *        a <code>Vector3D</code> specifying the tangent axisTwo of the surfel
	 * @param radius
	 *        the radius of the surfel
	 * @param diffuseColor
	 *        the <code>QRgb</code> value of the diffuse color of the surfel
	 * @param specularColor
	 *        the <code>QRgb</code> value of the specular color of the surfel
	 * @param flags
	 *        the <code>Flags</code> of the surfel
	 * @param ambientCoefficient
	 *        the ambient coefficient of the surfel; must be in [0.0, 1.0]
	 * @param diffuseCoefficient
	 *        the diffuse coefficient of the surfel; must be in [0.0, 1.0]
	 * @param specularCoefficient
	 *        the specular coefficient of the surfel; must be in [0.0, 1.0]
	 * @param shininess
	 *        the shininess of the surfel; must be in [0.0, 1.0]
	 * @param textureCoordinate
	 *        the <code>TextureCoordinate</code> which contains the UV texture coordinates
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @return a pointer to the newly inserted <code>SurfelInterface</code>
	 * @see #surfelCollectionChanged
	 */
	CORE_API SurfelInterface *addSurfel (const Vector3D position, const Vector3D normal,
		                                 const Vector3D axisOne, const Vector3D axisTwo,
		                                 const float radius, const QRgb diffuseColor, const QRgb specularColor, 
								         const SurfelInterface::Flags flags = SurfelInterface::NO_FLAGS,
									     const float ambientCoefficient = 0.5f, const float diffuseCoefficient = 0.75f,
									     const float specularCoefficient = 0.25f, const float shininess = 0.75,
										 const MyDataTypes::TextureCoordinate textureCoordinate = defaultTextureCoordinate,
								         const bool emitSignal = true);
	/**
	 * Adds a new <code>SurfelInterface</code> with default properties to this <code>SurfelCollection</code> and
	 * returns a pointer to it.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @return a pointer to the newly inserted <code>SurfelInterface</code>
	 * @see #surfelCollectionChanged
	 */
	CORE_API SurfelInterface *addSurfel (const bool emitSignal = true);

	/**
	 * Adds a <code>SurfelInterface</code> to this <code>SurfelCollection</code> by value.
	 * 
	 * <b>Note</b>: a copy of the surfel is generated and added to this <code>SurfelCollection</code>, making
	 *              sure that the <code>SurfelInterface</code>s within this <code>SurfelCollection</code>
	 *              have the same type.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfel
	 *        a pointer to an existing <code>SurfelInterface</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @return a pointer to the newly inserted <code>SurfelInterface</code>
	 * @see #surfelCollectionChanged
	 */
	CORE_API SurfelInterface *addSurfel (const SurfelInterface *surfel, const bool emitSignal = true);

	/**
	 * Add surfels to this <code>SurfelCollection</code> by value.
	 *
	 * <b>Note</b>: a copy of each surfel is generated and added to this <code>SurfelCollection</code>, making
	 *              sure that the <code>SurfelInterface</code>s within this <code>SurfelCollection</code>
	 *              have the same type.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfels
	 *        a pointer to a <code>QVector</code> containing all the <code>SurfelInterface</code>s
	 *        to be added to this <code>SurfelCollection</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #surfelCollectionChanged
	 */
	CORE_API void addSurfels (const QVector<SurfelInterface> *surfels, const bool emitSignal = true);

	/**
	 * Adds the <code>SurfelInterface</code>s in <code>surfels</code> to this <code>SurfelCollection</code>.
	 * The <code>SurfelInterface</code>s in <code>surfels</code> must be of the same type as the
	 * <code>SurfelInterface</code>s in this <code>SurfelCollection</code>.
	 *
	 * <b>Attention</b>: The <code>SurfelCollection</code> <code>surfels</code> is invalidated by this operation!
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfels
	 *        a pointer to a <code>SurfelCollection</code> which contains the <code>SurfelInterface</code> to
	 *        be added to this <code>SurfelCollection</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #surfelCollectionChanged
	 */
	CORE_API void merge (const SurfelCollection *surfels, const bool emitSignal = true);

	/**
	 * Removes the current <code>SurfelInterface</code> in this <code>SurfelCollection</code> according
	 * to the internal iterator.
	 */
	CORE_API void removeCurrentSurfel();
	
	/**
	 * Removes the <code>surfel</code> from this <code>SurfelCollection</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfel
	 *        a pointer to a <code>SurfelInterface</code> to be removed from this <code>SurfelCollection</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @return <code>true</code> if the <code>surfel</code> could be removed; <code>false</code> else
	 * @see #surfelCollectionChanged
	 */
	CORE_API bool removeSurfel (SurfelInterface *surfel, const bool emitSignal = true);

	/**
	 * Removes the <code>surfels</code> from this <code>SurfelCollection</code>. 
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfels
	 *        a pointer to a <code>QList<SurfelInterface></code> which contains the <code>SurfelInterface</code>s
	 *        to be removed from this <code>SurfelCollection</code>;
	 *
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #surfelCollectionChanged
	 */
	CORE_API void removeSurfels (const QList<SurfelInterface> *surfels, const bool emitSignal = true);


	/**
	 * Removes the <code>surfels</code> from this <code>SurfelCollection</code>. 
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param surfels
	 *        a pointer to a <code>QArray<SurfelInterface></code> which contains the <code>SurfelInterface</code>s
	 *        to be removed from this <code>SurfelCollection</code>;
	 *
	 * @param nofSurfels
	          number of surfels in <code>surfels</code>
     *
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #surfelCollectionChanged
	 */
	CORE_API void removeSurfels (const QArray<SurfelInterface *> surfels, const int nofSurfels, const bool emitSignal = true);
	
	/**
	 * Makes a copy of the <code>sourceCollection</code>. All surfels contained in this <code>SurfelCollection</code>
	 * are removed and <code>delete</code>d before copying.
	 *
	 * @param sourceCollection
	 *        a pointer to the source <code>SurfelCollection</code> which is to be copied
	 *        into this <code>SurfelCollection</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 */
	CORE_API void copy (SurfelCollection *sourceCollection, const bool emitSignal = true);

	/**
	 * Returns a copy of this <code>Object</code>. The copy must be <code>delete</code>d after use.
	 *
	 * @return a copy of this <code>Object</code>; must be <code>delete</code>d after use
	 */
	CORE_API SurfelCollection *copy();

	/**
	 * Clears all <code>SurfelInterface</code>s from this <code>SurfelCollection</code>.
	 *
	 * <b>Signals:</b>
	 * <ul>
	 *   <li><code>surfelCollectionChanged</code></li>
	 * </ul>
	 *
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #surfelCollectionChanged
	 */
	CORE_API void clear (const bool emitSignal = true);

signals:

	/**
	 * Emitted whenever the <code>SurfelCollection</code> has been changed.
	 *
	 * <b>Note</b>: Renderers should not connect to this <code>surfelCollectionChanged</code> signal directly,
	 *              they should connect to the signal <code>Scene::objectModified</code> instead, since
	 *              internally the <code>Object</code> already connects to this signal, which on its turn informs
	 *              the <code>Scene</code>.
	 * @see Scene#objectModified
	 */
	void surfelCollectionChanged();

	/**
	 * Emitted whenever the type of the surfels in this <code>SurfelCollection</code> has been changed.
	 *
	 * <b>Note</b>: Renderers should not connect to this <code>surfelTypeChanged</code> signal directly,
	 *              they should connect to the signal <code>Scene::objectModified</code> instead, since
	 *              internally the <code>Scene</code> already connects to this signal, which on its turn
	 *              converts all the <code>SurfelCollection</code>s of the other <code>Object</code>s
	 *              in the <code>Scene</code>.
	 * @see Scene#objectobjectModified
	 */
	void surfelTypeChanged (const SurfelInterface::PropertyDescriptor newPropertyDescriptor);

private:

	SurfelInterface::PropertyDescriptor propertyDescriptor;   // the current type of surfels in this collection
	SurfelList surfels;		                                  // the list of surfels. a custom list class

};


#endif  // __SURFELCOLLECTION_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
