//
// Title:   SurfelList.cpp
// Created: Mon Sep 23 13:47:04 2002
// Authors: Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "SurfelList.h"

SurfelList::SurfelList () {

	firstSurfel = 0;
	lastSurfel = 0;
	currentSurfel = 0;
	nSurfels = 0;
}

SurfelList::~SurfelList () {

	clear();
}

uint SurfelList::count () const {

	return nSurfels;
}

bool SurfelList::isEmpty () const {

	return firstSurfel ? true : false;
}

void SurfelList::setAutoDeleteEnabled(bool enable) {

	autoDelete = enable;
}

void SurfelList::append (SurfelInterface* surfel) {

	nSurfels++;

	if (lastSurfel == 0) {
		surfel->next = 0;
		surfel->prev = 0;

		firstSurfel = surfel;
	} 
	else {
		lastSurfel->next = surfel;
		surfel->prev = lastSurfel;
		surfel->next = 0;
	}
	lastSurfel = surfel;

	currentSurfel = surfel;
}


void SurfelList::append (const SurfelList* surfelList) {

	nSurfels += surfelList->count();

	if(lastSurfel == 0) {
		firstSurfel = surfelList->firstSurfel;
	} 
	else {
		lastSurfel->next = surfelList->firstSurfel;
		surfelList->firstSurfel->prev = lastSurfel;
	}

	lastSurfel = surfelList->lastSurfel;
	currentSurfel = lastSurfel;
}


bool SurfelList::remove (SurfelInterface* surfel) {

	nSurfels--;

	if(surfel == firstSurfel) {
		firstSurfel = surfel->next;
		currentSurfel = surfel->next;

		if(surfel == lastSurfel) {
			lastSurfel = 0;
		}
		else {
			firstSurfel->prev = 0;
		}
	} 
	else {
		surfel->prev->next = surfel->next;

		if(surfel == lastSurfel) {
			lastSurfel = surfel->prev;
			lastSurfel->next = 0;
			currentSurfel = lastSurfel;
		}
		else {
			surfel->next->prev = surfel->prev;
			currentSurfel = surfel->next;
		}
	}

	if (autoDelete == true) {
		surfel->destroy();
	}

	return true;
}

bool SurfelList::remove () {

	if(currentSurfel) {
		return remove(currentSurfel);
	}
	else {
		return false;
	}
}

bool SurfelList::removeFirst () {

	SurfelInterface *t;

	if(firstSurfel) {
		t = firstSurfel;
		nSurfels--;

		if(firstSurfel == lastSurfel) {
			firstSurfel = 0;
			lastSurfel = 0;
			currentSurfel = 0;
		} 
		else {
			firstSurfel = firstSurfel->next;
			firstSurfel->prev = 0;
			currentSurfel = firstSurfel;
		}

		if(autoDelete == true) {
			t->destroy();
		}

		return true;
	}
	else {
		return false;
	}

}

bool SurfelList::removeLast () {

	SurfelInterface *t;

	if(lastSurfel) {
		t = lastSurfel;
		nSurfels--;

		if(lastSurfel == firstSurfel) {
			firstSurfel = 0;
			lastSurfel = 0;
			currentSurfel = 0;
		}
		else {
			lastSurfel = lastSurfel->prev;
			lastSurfel->next = 0;
			currentSurfel = lastSurfel;
		}

		if (autoDelete == true) {
			t->destroy();
		}

		return true;
	}
	else {
		return false;
	}
			
}

void SurfelList::clear () {

	SurfelInterface *surfel, *t;

	surfel = firstSurfel;
	while(surfel != 0) {
		t = surfel;
		surfel = surfel->next;

		if (autoDelete == true) {
			t->destroy();
		}
	}

	firstSurfel = 0;
	lastSurfel = 0;
	currentSurfel = 0;
	nSurfels = 0;
}

SurfelInterface* SurfelList::current () const {

	return currentSurfel;
}

SurfelInterface* SurfelList::getFirst () const {

	return firstSurfel;
}

SurfelInterface* SurfelList::getLast () const {

	return lastSurfel;
}

SurfelInterface* SurfelList::first () {

	currentSurfel = firstSurfel;
	return firstSurfel;
}

SurfelInterface* SurfelList::last () {

	currentSurfel = lastSurfel;
	return lastSurfel;
}

SurfelInterface* SurfelList::next () {

	if(currentSurfel->next) {
		currentSurfel = currentSurfel->next;
		return currentSurfel;
	} 
	else {
		return 0;
	}
}

SurfelInterface* SurfelList::prev () {

	if(currentSurfel->prev) {
		currentSurfel = currentSurfel->prev;
		return currentSurfel;
	}
	else {
		return 0;
	}
}

int SurfelList::find (const SurfelInterface *surfel) {

	SurfelInterface *cur;
	int i;

	cur = firstSurfel;
	i = 0;
	while (cur != 0 && cur != surfel) {
		cur = cur->next;
		i++;
	}

	if (cur != 0) {
		currentSurfel = cur;
		return i;
	}
	else {
		return -1;
	}
}

void SurfelList::toVector ( QVector<SurfelInterface> * vec ) const {

	SurfelInterface* surfel;
	int i;

	vec->resize(nSurfels);


	surfel = firstSurfel;
	i = 0;
	while(surfel != 0) {
		vec->insert(i, surfel);
		i++;
		surfel = surfel->next;
	}
	
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
