//
// Title:   SurfelInterface.h
// Created: Mon Sep 23 13:43:35 2002
// Authors: Oliver Knoll
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SURFELINTERFACE_H_
#define __SURFELINTERFACE_H_

#include <qcolor.h>
#include <qstring.h>
#include <qstringlist.h>

#include "../../../DataTypes/src/MyDataTypes.h"
#include "../../../DataTypes/src/Vector3D.h"
#include "../../src/CoreDLL.h"

/**
 * This interface defines the API for surfels, surface elements, which are the atomic elements of
 * which <code>Object</code>s consist of (instead of triangles). Think of them as points
 * with a normal, which defines the orientation of the circle (or ellipse) which is drawn
 * as soon as the point (surfel) is rendered.
 *
 * This interface provides some static methods for converting between user- and system readable
 * surfel properties.
 *
 * <b>Note:</b> only the <code>SurfelCollection</code> class is allowed to create and <code>delete</code>
 *              individual surfels (<code>addSurfel</code> and <code>removeSurfel</code>) and change their
 *              position, so that the <code>SurfelCollection</code> can maintain a consistent state
 *              (for search-structures and the like). Additionally each property can only be set via
 *              the <code>Scene</code> so renderers can be informed accordingly.
 *
 * @author Oliver Knoll
 * @version 1.2
 */
class SurfelInterface {

public:

	/**
	 * Properties which can be OR'ed together. Up to 32 properties are supported.
	 */
	typedef enum property {

		NO_PROPERTIES        = 0x00000000,  // no properties
		POSITION			 = 0x00000001,  // the position of the surfel
		NORMAL				 = 0x00000002,	// the normal of the surfel
		RADIUS				 = 0x00000004,  // the radius of the surfel
		DIFFUSE_COLOR		 = 0x00000008,	// the diffuse color of the surfel
		SPECULAR_COLOR		 = 0x00000010,	// the specular color of the surfel
		FLAGS                = 0x00000020,	// the flags of the surfel
		AMBIENT_COEFFICIENT  = 0x00000040,  // the ambient coefficient 
		DIFFUSE_COEFFICIENT  = 0x00000080,  // the diffuse coefficient
		SPECULAR_COEFFICIENT = 0x00000100,  // the specular coefficient
		SHININESS            = 0x00000200,  // the shininess
		TEXTURE_COORDINATE   = 0x00000400,  // the texture UV coordinates
		TANGENT_AXES         = 0x00000800,  // the tangent axes
		ALL_PROPERTIES		 = 0xffffffff	// all properties

	} Property;

	/**
	 * Consists of up to 32 OR'ed <code>Property</code> values, describing which properties
	 * of a <code>SurfelInterface</code> are supported/changed.
	 *
	 * @see #Property
	 */
	typedef int PropertyDescriptor;

	/**
	 * Flags which can be OR'ed together - add new flag values here. Up to 32 flags are supported.
	 */
	typedef enum flag {

		NO_FLAGS    = 0x00000000,   // no flags set
		SELECTED1	= 0x00000001,   // set when this surfel is part of selection 1 - does NOT render the surfel differently
		                            // set the flag EMPHASISE as well in this case
		SELECTED2	= 0x00000002,   // set when this surfel is part of selection 2 - does NOT render the surfel differently
		                            // set the flag EMPHASISE as well in this case
		SELECTED3	= 0x00000004,   // set when this surfel is part of selection 3 - does NOT render the surfel differently
		                            // set the flag EMPHASISE as well in this case
		EMPHASISE	= 0x00000008,   // set when the surfel should be rendered differently, e.g. when selected
		COVERED		= 0x00000010,   // used internally (only for temporary computations!)
		ALL_FLAGS	= 0xffffffff

	} Flag;

	/**
	 * Consists of up to 32 OR'ed <code>Flag</code> values.
	 *
	 * @see #Flag
	 */
	typedef int Flags;

	/**
	 * Returns a user readable <code>QString</code> for the <code>property</code>.
	 *
	 * @param property
	 *        the <code>Property</code> to be translated into a user-readable <code>QString</code>
	 * @return a <code>QString</code> containing a user-readable text for the <code>property</code>;
	 *         a default value is returned in case the <code>property</code> isn't supported yet
	 * @see Property
	 */
	CORE_API static QString getUserReadableProperty (const Property property);

	/**
	 * Returns a "system" readable <code>Property</code> for the user-readable <code>QString</code>.
	 *
	 * @param userReadableProperty
	 *        a <code>QString</code> containing a user-readable text for a <code>Property</code>
	 * @return a "system" readable <code>Property</code>; in case the <code>userReadableProperty</code>
	 *         cannot be mapped to one of the system <code>Properties</code> <code>NO_PROPERTIES</code> is
	 *         returned
	 * @see Property
	 */
	CORE_API static Property getSystemProperty (const QString userReadableProperty);

	/**
	 * Returns a <code>QStringList</code> which contains user readable <code>QString</code>s for
	 * all editable <code>Properties</code> which is useful for <code>QComboBox</code>es, for example.
	 * The entries in this <code>QStringList</code> can be mapped to system <code>Properties</code>
	 * with <code>getSystemProperty</code>.
	 *
	 * @return a <code>QStringList</code> containing user readable <code>QString</code>s for all
	 *         editable <code>Properties</code>
	 * @see #getSystemProperty
	 */
	CORE_API static QStringList getEditableProperties();

	/**
	 * Returns a property descriptor that only lists the editable properties.
	 *
	 * @return a property descriptor that only lists the editable properties
	 */
	CORE_API static PropertyDescriptor getEditablePropertyDescriptor();

	/**
	 * Returns the number of editable properties.
	 *
	 * @return the number of editable properties
	 */
	CORE_API static int getNumOfEditableProperties();

	/**
	 * Returns the number of data components that should be provided by a brush to paint given
	 * a surfel property (e.g., 1 for scalar attributes,3 for RGB color).
	 * 
	 * <b>Note:</b> this method is only useful for properties that can be painted by a brush!
	 *
	 * @return the number of data components that should be provided by a brush to paint
	 */
	CORE_API static int getNumOfPropertyBrushComponents(Property property);

	/**
     * Returns the number of supported selections.
	 *
	 * @return the number of supported selections
	 */
	CORE_API static int getNumOfSelections();
	
	/**
     * Returns all selection flags OR'ed together.
	 *
	 * @return all selection flags OR'ed together
	 */
	CORE_API static Flags getSelectionFlags();

	/**
	 * Returns the position.
	 *
	 * @return a <code>Vector3D</code> containing the position
	 * @see Scene#setSurfelPosition
	 */
    virtual Vector3D getPosition() const = 0;

	/**
	 * Returns the normal.
	 *
	 * @return a <code>Vector3D</code> containing the normal
	 * @see Scene#setSurfelNormal
	 */
	virtual Vector3D getNormal() const = 0;	

	/**
	 * Returns the axis 1.
	 *
	 * @return a <code>Vector3D</code> containing the axis 1
	 * @see Scene#setSurfelTangentAxes
	 */
	virtual Vector3D getAxisOne() const = 0;	

	/**
	 * Returns the axis 2.
	 *
	 * @return a <code>Vector3D</code> containing the axis 2
	 * @see Scene#setSurfelTangentAxes
	 */
	virtual Vector3D getAxisTwo() const = 0;	

	/**
	 * Returns the radius.
	 *
	 * @return the radius
	 * @see Scene#setSurfelRadius
	 */
	virtual float getRadius() const = 0;	

	/**
	 * Returns the diffuse color.
	 *
	 * @return the <code>QRgb</code> diffuse color
	 * @see Scene#setSurfelDiffuseColor
	 */
	virtual QRgb getDiffuseColor() const = 0;

	/**
	 * Returns the specular color.
	 *
	 * @return the <code>QRgb</code> specular color
	 * @see Scene#setSurfelSpecularColor
	 */
	virtual QRgb getSpecularColor() const = 0;

	/**
	 * Returns <code>true</code> if <em>all</em> flags specified in <code>flags</code>
	 * are set to <code>true</code>.
	 *
	 * @param flags
	 *        OR'ed <code>Flag</code> values which specify the <code>Flags</code> to be tested
	 * @return <code>true</code> if <em>all</em> flags specified in <code>flags</code>
	 *         are set to <code>true</code>; <code>false</code> else
	 * @see Scene#setSurfelFlagOn
	 */
	virtual bool isFlagOn (const Flags flags) const = 0;

	/**
	 * Returns <code>true</code> if this <code>SurfelInterface</code> is selected.
	 *
	 * @return <code>true</code> if this <code>SurfelInterface</code> is selected; <code>false</code> else
	 * @see Scene#setSurfelSelected
	 */ 
	virtual bool isSelected() const = 0;

	/**
	 * Returns the <code>Flags</code>.
	 *
	 * @return the <code>Flags</code>
	 * @see Scene#setSurfelFlags
	 */
	virtual Flags getFlags() const = 0;

	/**
	 * Returns the ambient coefficient.
	 *
	 * @return the ambient coefficient
	 * @see Scene#setSurfelAmbientCoefficient
	 */
	virtual float getAmbientCoefficient() const = 0;

	/**
	 * Returns the diffuse coefficient.
	 *
	 * @return the diffuse coefficient
	 * @see Scene#setSurfelDiffuseCoefficient
	 */
	virtual float getDiffuseCoefficient() const = 0;

	/**
	 * Returns the specular coefficient.
	 *
	 * @return the specular coefficient
	 * @see Scene#setSurfelSpecularCoefficient
	 */
	virtual float getSpecularCoefficient() const = 0;

	/**
	 * Returns the shininess.
	 *
	 * @return the shininess
	 * @see Scene#setSurfelShininess
	 */
	virtual float getShininess() const = 0;

	/**
	 * Returns the <code>TextureCoordinate</code>.
	 *
	 * @return the <code>TextureCoordinate</code>
	 * @see Scene#setSurfelTextureCoordinate
	 */
	virtual MyDataTypes::TextureCoordinate getTextureCoordinate() const = 0;

	/**
	 * Returns a deep copy of this <code>SurfelInterface</code>. The copy must be <code>delete</code>d after use.
	 *
	 * @return a deep copy of this <code>SurfelInterface</code>; must be <code>delete</code>d after use
	 */
	virtual SurfelInterface *copy() const = 0;

protected:

	// only the Scene may change these properties, so it can maintain a consistent
	// state and emit the signal 'objectModified' (we don't want that each surfel itself
	// sends signals)
	friend class Scene;
	friend class SurfelCollection;   // for readFromStream, writeToStream
	friend class SurfelList;		 // for access to prev, next pointers

	/**
	 * Since surfels have their own memory management, they have to be destroyed by calling
	 * this <code>destroy</code> method instead of the standard <code>delete</code> operator.
	 */
	virtual void destroy() = 0;

	/**
	 * Sets the position.
	 *
	 * @param newPosition
	 *        a <code>Vector3D</code> containing the new position
	 */
	virtual void setPosition (const Vector3D newPosition) = 0;

	/**
	 * Sets the normal.
	 *
	 * @param newNormal
	 *        a <code>Vector3D</code> containing the new normal
	 */
	virtual void setNormal (const Vector3D newNormal) = 0;

	/**
	 * Sets the axis 1.
	 *
	 * @param newAxisOne
	 *        a <code>Vector3D</code> containing the new axis 1
	 */
	virtual void setAxisOne (const Vector3D newAxisOne) = 0;

	/**
	 * Sets the axis 2.
	 *
	 * @param newAxisTwo
	 *        a <code>Vector3D</code> containing the new axis 2
	 */
	virtual void setAxisTwo (const Vector3D newAxisTwo) = 0;

	/**
	 * Sets the radius.
	 *
	 * @param newRadius
	 *        the new radius
	 */
	virtual void setRadius (const float newRadius) = 0;

	/**
	 * Sets the diffuse color.
	 *
	 * @param newDiffuseColor
	 *        the new <code>QRgb</code> diffuse color
	 */
	virtual void setDiffuseColor (const QRgb newDiffuseColor) = 0;

	/**
	 * Sets the specular color.
	 *
	 * @param newSpecularColor
	 *        the new <code>QRgb</code> specular color
	 */
	virtual void setSpecularColor (const QRgb newSpecularColor) = 0;

	/**
	 * Sets <em>all</em> the flags specified in <code>flags</code> to the value <code>on</code>.
	 * 
	 * @param flags
	 *        specifies which flags are to be set/unset
	 * @param on
	 *        the value of each flag to be set/unset
	 */
	virtual void setFlagOn (const Flags flags, const bool on) = 0;

	/**
	 * Sets the flags of this <code>SurfelInterface</code> to <code>newFlags</code>.
	 *
	 * @param newFlags
	 *        the new <code>Flags</code> of this <code>SurfelInterface</code>
	 */
	virtual void setFlags (const Flags newFlags) = 0;

	/**
	 * Sets the ambient coefficient.
	 *
	 * @param newAmbientCoefficient
	 *        the new ambient coefficient
	 */
	virtual void setAmbientCoefficient (const float newAmbientCoefficient) = 0;

	/**
	 * Sets the diffuse coefficient.
	 *
	 * @param newDiffuseCoefficient
	 *        the new diffuse coefficient
	 */
	virtual void setDiffuseCoefficient (const float newDiffuseCoefficient) = 0;

	/**
	 * Sets the specular coefficient.
	 *
	 * @param newSpecularCoefficient
	 *        the new specular coefficient
	 */
	virtual void setSpecularCoefficient (const float newSpecularCoefficient) = 0;

	/**
	 * Sets the shininess.
	 *
	 * @param newShininess
	 *        the new shininess
	 */
	virtual void setShininess (const float newShininess) = 0;

	/**
	 * Sets the texture coordinate.
	 *
	 * @param newTextureCoordinate
	 *        the new <code>TextureCoordinate</code>
	 */
	virtual void setTextureCoordinate (const MyDataTypes::TextureCoordinate newTextureCoordinate) = 0;

	/**
	 * Copies the attributes of the <code>sourceSurfel</code> to this surfel.
	 *
	 * @param sourceSurfel
	 *        a pointer to the source <code>SurfelInterface</code> which is to be copied
	 *        into this <code>SurfelInterface</code>
	 * @param emitSignal
	 *        set to <code>false</code> if the signal <code>surfelCollectionChanged</code> should
	 *        be supressed; useful if multiple operations have to be done before the image gets
	 *        re-rendered; use with <b>caution</b>
	 * @see #setAutoDelete
	 */
	virtual void copy (const SurfelInterface *sourceSurfel) = 0;

	/**
	 * the previous <code>SurfelInterface</code>. <code>SurfelInterface</code>s are connected to a double linked list. 
	 */
	SurfelInterface* prev;

	/**
	 * the next <code>SurfelInterface</code>. <code>SurfelInterface</code>s are connected to a double linked list. 
	 */
	SurfelInterface* next;

};


#endif  // __SURFELINTERFACE_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
