//
// Title:   Painter.cpp
// Created: Mon Sep 23 13:40:27 2002
// Authors: Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "../../Scene/src/Scene.h"
#include "../../../Utilities/src/Common.h"
#include "../../../Utilities/src/MyMathTools.h"
#include "../../DataStructures/src/SurfelCollection.h"

#include "Painter.h"




Painter::Painter (const Brush *brush) {

	scene = Scene::getInstance();
	filterBlendingGrid = 0;
	this->setBrush (brush);

}


Painter::~Painter() {

	if (filterBlendingGrid) {
		delete[] filterBlendingGrid;
	}
	
}

bool Painter::paint (QArray<SurfelInterface*> surfels,
					 int numSurfels,
					 const QArray<MyDataTypes::TextureCoordinate> texCoords,
					 const float scale,
					 const Vector3D *carveDirection,
					 const float maxDeviation,
					 const bool gaps) {

    (void)maxDeviation;

	// assumes that splat coordinates are brush coordinates!!
	
	int              i; 
	float            alpha;
	
	Vector3D         oldPosition,
		             displacementVector,
		             newPosition;	
	float            displacementScalar;
		
	QRgb             oldColor,
		             newColor;		
	float            red,
		             green,
		             blue;
	
	float            oldCoeff,
		             newCoeff;
	
	

	/////////////////////////////////////////////////////////////////////////////
	// with resampling, surfels are arranged on regular grid
	//

	if (!texCoords) {

		// if there are gaps in the grid, certain surfels in the grid are 0 pointers. therefore,
		// the expected radius has to be computed otherwise! TODO!
		if(!gaps) {
			// compute the expected average radius by 
			float topRowLength = (surfels[0]->getPosition() - surfels[sizeX - 1]->getPosition()).getLength();
			float bottomRowLength = (surfels[(sizeY - 1) * sizeX]->getPosition() - surfels[sizeY * sizeX - 1]->getPosition()).getLength();
			float leftColumnLength = (surfels[0]->getPosition() - surfels[(sizeY - 1) * sizeX]->getPosition()).getLength();
			float rightColumnLength = (surfels[sizeX - 1]->getPosition() - surfels[sizeY * sizeX - 1]->getPosition()).getLength();
			
			expectedRadius = MyMathTools::maxf (MyMathTools::maxf (topRowLength, bottomRowLength) / (float) sizeX, MyMathTools::maxf (leftColumnLength, rightColumnLength) / (float) sizeY);
		}
		surfelRadiusTooBig = false;
		

		// assumes that splats are arranged in grid according to brush resolution!
		int x, y;

		// positions
		if (displacementChannel) {			
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {
					
					if(surfels[y * sizeX + x]) {
						oldPosition = surfels[y * sizeX + x]->getPosition();
						
						if (displacementAlpha) {
							displacementScalar = displacementAlpha->getValue (x, y);
						}
						else {
							displacementScalar = 1.0f;
						}					
						if (carveDirection) {
							// assumes that carving component stores distance of surface point to tool reference plane!
							if (scale > 0.0f) {
								displacementScalar *= MyMathTools::maxf (0.0f, scale * displacementNormal->getValue (x, y) - displacementCarving->getValue (x, y));						
							}
							else {
								displacementScalar *= -MyMathTools::maxf (0.0f, -(scale * displacementNormal->getValue (x, y) - displacementCarving->getValue (x, y)));						
							}
							displacementVector = displacementScalar * (*carveDirection);
						}
						else {
							displacementVector = (scale * displacementScalar * displacementNormal->getValue (x, y)) * surfels[y * sizeX + x]->getNormal();
						}
					
						scene->setSurfelPosition (surfels[y * sizeX + x], oldPosition + displacementVector, false);
					}
				}
			}
		}			

		if(!gaps) {
			if (!computeRadiiAndNormals (surfels)) {
				return false;
			}
		} 
		else {
			if (!computeRadiiAndNormalsWithGaps (surfels)) {
				return false;
			}
		}


		// diffuse color
		if (diffColorChannel) {
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {

					if(surfels[y * sizeX + x]) {
						red = 255.0f * diffRedComponent->getValue (x, y);
						green = 255.0f * diffGreenComponent->getValue (x, y);
						blue = 255.0f * diffBlueComponent->getValue (x, y);
						
						if (diffColorAlpha) {
							oldColor = surfels[y * sizeX + x]->getDiffuseColor ();
							alpha = diffColorAlpha->getValue (x, y);
							red   = alpha * red   + (1 - alpha) * qRed (oldColor);
							green = alpha * green + (1 - alpha) * qGreen (oldColor);
							blue  = alpha * blue  + (1 - alpha) * qBlue (oldColor);
						}					
						newColor = qRgb ((int)red, (int)green, (int)blue);

						scene->setSurfelDiffuseColor (surfels[y * sizeX + x], newColor, false);
					}
				}
			}
		}

		
		// specular color
		if (specColorChannel) {
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {
															
					if(surfels[y * sizeX + x]) {
						red = 255.0f * specRedComponent->getValue (x, y);
						green = 255.0f * specGreenComponent->getValue (x, y);
						blue = 255.0f * specBlueComponent->getValue (x, y);
						
						if (specColorAlpha) {
							oldColor = surfels[y * sizeX + x]->getSpecularColor ();
							alpha = specColorAlpha->getValue (x, y);
							red   = alpha * red   + (1 - alpha) * qRed (oldColor);
							green = alpha * green + (1 - alpha) * qGreen (oldColor);
							blue  = alpha * blue  + (1 - alpha) * qBlue (oldColor);
						}					
						newColor = qRgb ((int)red, (int)green, (int)blue);					

						scene->setSurfelSpecularColor (surfels[y * sizeX + x], newColor, false);
					}
				}
			}
		}


		// diffuse coefficient
		if (diffCoeffChannel) {
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {

					if(surfels[y * sizeX + x]) {															
						if (diffuseCoefficientAlpha) {
							alpha = diffuseCoefficientAlpha->getValue (x, y);
							oldCoeff = surfels[y * sizeX + x]->getDiffuseCoefficient ();
							newCoeff = alpha * diffuseCoefficient->getValue (x, y) + (1 - alpha) * oldCoeff;	
						}
						else {
							newCoeff = diffuseCoefficient->getValue (x, y);
						}

						scene->setSurfelDiffuseCoefficient (surfels[y * sizeX + x], newCoeff, false);					
					}
				}
			}
		}


		// specular coefficient
		if (specCoeffChannel) {
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {

					if(surfels[y * sizeX + x]) {															
						if (specularCoefficientAlpha) {
							alpha = specularCoefficientAlpha->getValue (x, y);
							oldCoeff = surfels[y * sizeX + x]->getSpecularCoefficient ();
							newCoeff = alpha * specularCoefficient->getValue (x, y) + (1 - alpha) * oldCoeff;	
						}
						else {
							newCoeff = specularCoefficient->getValue (x, y);
						}

						scene->setSurfelSpecularCoefficient (surfels[y * sizeX + x], newCoeff, false);
					}
				}
			}
		}


		// shininess
		if (shininessChannel) {
			for (x = 0; x < sizeX; x++) {
				for (y = 0; y < sizeY; y++) {
					
					if(surfels[y * sizeX + x]) {
						if (shininessAlpha) {
							alpha = shininessAlpha->getValue (x, y);
							oldCoeff = surfels[y * sizeX + x]->getShininess ();
							newCoeff = alpha * shininess->getValue (x, y) + (1 - alpha) * oldCoeff;	
						}
						else {
							newCoeff = shininess->getValue (x, y);
						}
					
						scene->setSurfelShininess (surfels[y * sizeX + x], newCoeff, false);
					}
				}
			}
		}

	}

	/////////////////////////////////////////////////////////////////////////////
	// sampling on old positions, texture coordinates are provided
	// 
	else {

		// position
		// not supported yet!!


		// diffuse color		
		if (diffColorChannel) {
			for (i = 0; i < numSurfels; i++) {
				
				
				red = 255.0f * diffRedComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				green = 255.0f * diffGreenComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				blue = 255.0f * diffBlueComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				
				if (diffColorAlpha) {
					oldColor = surfels[i]->getDiffuseColor ();
					alpha = diffColorAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					red   = alpha * red   + (1 - alpha) * qRed (oldColor);
					green = alpha * green + (1 - alpha) * qGreen (oldColor);
					blue  = alpha * blue  + (1 - alpha) * qBlue (oldColor);
				}
				
				newColor = qRgb ((int)red, (int)green, (int)blue);
				
				scene->setSurfelDiffuseColor (surfels[i], newColor, false);
			}
		}


		// specular color		
		if (specColorChannel) {
			for (i = 0; i < numSurfels; i++) {
				
				
				red = 255.0f * specRedComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				green = 255.0f * specGreenComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				blue = 255.0f * specBlueComponent->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				
				if (specColorAlpha) {
					oldColor = surfels[i]->getSpecularColor ();
					alpha = specColorAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					red   = alpha * red   + (1 - alpha) * qRed (oldColor);
					green = alpha * green + (1 - alpha) * qGreen (oldColor);
					blue  = alpha * blue  + (1 - alpha) * qBlue (oldColor);
				}
				
				newColor = qRgb ((int)red, (int)green, (int)blue);
				
				scene->setSurfelSpecularColor (surfels[i], newColor, false);
			}
		}

		// ambient Coefficient
		if (ambCoeffChannel) {
			for (i = 0; i < numSurfels; i++) {
					
				if (ambientCoefficientAlpha) {
					oldCoeff = surfels[i]->getAmbientCoefficient ();
					alpha = ambientCoefficientAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					newCoeff = alpha * ambientCoefficient->getValue ((int)texCoords[i].u,
																	 (int)texCoords[i].v) + (1 - alpha) * oldCoeff;	
				}
				else {
					newCoeff = ambientCoefficient->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				}
				scene->setSurfelAmbientCoefficient (surfels[i], newCoeff, false);
			}
		}

		// diffuse Coefficient
		if (diffCoeffChannel) {
			for (i = 0; i < numSurfels; i++) {
					
				if (diffuseCoefficientAlpha) {
					oldCoeff = surfels[i]->getDiffuseCoefficient ();
					alpha = diffuseCoefficientAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					newCoeff = alpha * diffuseCoefficient->getValue ((int)texCoords[i].u,
																	 (int)texCoords[i].v) + (1 - alpha) * oldCoeff;	
				}
				else {
					newCoeff = diffuseCoefficient->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				}
				scene->setSurfelDiffuseCoefficient (surfels[i], newCoeff, false);
			}
		}

		// specular Coefficient
		if (specCoeffChannel) {
			for (i = 0; i < numSurfels; i++) {					
				if (specularCoefficientAlpha) {
					oldCoeff = surfels[i]->getSpecularCoefficient ();
					alpha = specularCoefficientAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					newCoeff = alpha * specularCoefficient->getValue ((int)texCoords[i].u,
																	  (int)texCoords[i].v) + (1 - alpha) * oldCoeff;	
				}
				else {
					newCoeff = specularCoefficient->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				}
				scene->setSurfelSpecularCoefficient (surfels[i], newCoeff, false);
			}
		}

		// shininess
		if (shininessChannel) {
			for (i = 0; i < numSurfels; i++) {					
				if (shininessAlpha) {
					oldCoeff = surfels[i]->getShininess ();
					alpha = specularCoefficientAlpha->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
					newCoeff = alpha * 100.0f * shininess->getValue ((int)texCoords[i].u,
																	 (int)texCoords[i].v) + (1 - alpha) * oldCoeff;	
				}
				else {
					newCoeff = 100.0f * shininess->getValue ((int)texCoords[i].u, (int)texCoords[i].v);
				}
				scene->setSurfelShininess (surfels[i], newCoeff, false);
			}
		}

	}
	return true;
}






void Painter::setBrush (const Brush *newBrush) {

	sizeX = newBrush->getSize().width();
	sizeY = newBrush->getSize().height();

	/////////////////////////////////////////////////////////////////////////////
	displacementChannel = newBrush->getChannel (SurfelInterface::POSITION);
	if (displacementChannel) {
		
		displacementNormal  = displacementChannel->getComponent (0);	
		displacementCarving = displacementChannel->getComponent (1);	
		
		if (newBrush->hasAlphaChannel (SurfelInterface::POSITION)) {
			displacementAlpha = newBrush->getAlphaChannel (SurfelInterface::POSITION)->getComponent(0);	
		}
		else {
			displacementAlpha = 0;
		}			
	}
	
	/////////////////////////////////////////////////////////////////////////////
	diffColorChannel = newBrush->getChannel (SurfelInterface::DIFFUSE_COLOR);
	if (diffColorChannel) {
		
		if (diffColorChannel->getNOfComponents() == 3) {
			diffRedComponent = diffColorChannel->getComponent (0);
			diffGreenComponent = diffColorChannel->getComponent (1);
			diffBlueComponent = diffColorChannel->getComponent (2);
		}
		else {
			diffRedComponent = diffGreenComponent = diffGreenComponent = diffColorChannel->getComponent (0);
		}
		
		if (newBrush->hasAlphaChannel (SurfelInterface::DIFFUSE_COLOR)) {
			diffColorAlpha = newBrush->getAlphaChannel (SurfelInterface::DIFFUSE_COLOR)->getComponent(0);
		}
		else {
			diffColorAlpha = 0;
		}							
	}	

	/////////////////////////////////////////////////////////////////////////////
	specColorChannel = newBrush->getChannel (SurfelInterface::SPECULAR_COLOR);
	if (specColorChannel) {
		
		if (specColorChannel->getNOfComponents() == 3) {
			specRedComponent = specColorChannel->getComponent (0);
			specGreenComponent = specColorChannel->getComponent (1);
			specBlueComponent = specColorChannel->getComponent (2);
		}
		else {
			specRedComponent = specGreenComponent = specGreenComponent = specColorChannel->getComponent (0);
		}
		
		if (newBrush->hasAlphaChannel (SurfelInterface::SPECULAR_COLOR)) {
			specColorAlpha = newBrush->getAlphaChannel (SurfelInterface::DIFFUSE_COLOR)->getComponent(0);
		}
		else {
			specColorAlpha = 0;
		}							
	}	


	/////////////////////////////////////////////////////////////////////////////
	ambCoeffChannel = newBrush->getChannel (SurfelInterface::AMBIENT_COEFFICIENT);
	if (ambCoeffChannel) {
		
		ambientCoefficient  = ambCoeffChannel->getComponent (0);	
		
		if (newBrush->hasAlphaChannel (SurfelInterface::AMBIENT_COEFFICIENT)) {
			ambientCoefficientAlpha = newBrush->getAlphaChannel (SurfelInterface::AMBIENT_COEFFICIENT)->getComponent(0);	
		}
		else {
			ambientCoefficientAlpha = 0;
		}			
	}


	/////////////////////////////////////////////////////////////////////////////
	diffCoeffChannel = newBrush->getChannel (SurfelInterface::DIFFUSE_COEFFICIENT);
	if (diffCoeffChannel) {
		
		diffuseCoefficient  = diffCoeffChannel->getComponent (0);	
		
		if (newBrush->hasAlphaChannel (SurfelInterface::DIFFUSE_COEFFICIENT)) {
			diffuseCoefficientAlpha = newBrush->getAlphaChannel (SurfelInterface::DIFFUSE_COEFFICIENT)->getComponent(0);	
		}
		else {
			diffuseCoefficientAlpha = 0;
		}			
	}

	/////////////////////////////////////////////////////////////////////////////
	specCoeffChannel = newBrush->getChannel (SurfelInterface::SPECULAR_COEFFICIENT);
	if (specCoeffChannel) {
		
		specularCoefficient  = specCoeffChannel->getComponent (0);	
		
		if (newBrush->hasAlphaChannel (SurfelInterface::SPECULAR_COEFFICIENT)) {
			specularCoefficientAlpha = newBrush->getAlphaChannel (SurfelInterface::SPECULAR_COEFFICIENT)->getComponent(0);	
		}
		else {
			specularCoefficientAlpha = 0;
		}			
	}

	/////////////////////////////////////////////////////////////////////////////
	shininessChannel = newBrush->getChannel (SurfelInterface::SHININESS);
	if (shininessChannel) {
		
		shininess  = shininessChannel->getComponent (0);	
		
		if (newBrush->hasAlphaChannel (SurfelInterface::SHININESS)) {
			shininessAlpha = newBrush->getAlphaChannel (SurfelInterface::SHININESS)->getComponent(0);	
		}
		else {
			shininessAlpha = 0;
		}			
	}


	if (filterBlendingGrid) {
		delete[] filterBlendingGrid;
	}



	// compute the filter weight function for blending with original surface function
	// a boundary ring of size zeroBorder is zero
	// then a ring of size (border - borderZero) is blended towards 1
	// the inner ring is set to one

	filterBlendingGrid = new float [sizeX * sizeY];

	int x, y;
	float scaledX, scaledY;
	float border = 0.3f;
	float zeroBorder = 0.1f;
	float funX, funY;

	for (x = 0; x < sizeX; x++) {

		for (y = 0; y < sizeY; y++) {

			scaledX = (float) x / (float) sizeX;
			scaledY = (float) y / (float) sizeY;

			if ((scaledX <= zeroBorder) || (scaledX >= 1.0f - zeroBorder)) {
				funX = 0.0f;
			}
			else if (scaledX <= border) {				
				funX = 1.0f - (scaledX - zeroBorder) / (border - zeroBorder);
				funX = (funX * funX - 1.0f) * (funX * funX - 1.0f);
			} else if (scaledX >= 1.0f - border) {
				funX = (scaledX - (1.0f - border)) / (border - zeroBorder);
				funX = (funX * funX - 1.0f) * (funX * funX - 1.0f);
			} else {
				funX = 1.0f;
			}

			if ((scaledY <= zeroBorder) || (scaledY >= 1.0f - zeroBorder)) {
				funY = 0.0f;
			}
			else if (scaledY <= border) {				
				funY = 1.0f - (scaledY - zeroBorder) / (border - zeroBorder);
				funY = (funY * funY - 1.0f) * (funY * funY - 1.0f);
			} else if (scaledY >= 1.0f - border) {
				funY = (scaledY - (1.0f - border)) / (border - zeroBorder);
				funY = (funY * funY - 1.0f) * (funY * funY - 1.0f);
			} else {
				funY = 1.0f;
			}


			filterBlendingGrid [y * sizeX + x] = funY * funX;

		}	
		
	}

}


bool Painter::computeRadiiAndNormals (QArray<SurfelInterface *> surfels) {

	int x, y;
	Vector3D dX, dY;
	
	for (x = 1; x < sizeX - 1; x++) {
		// top row
		dX = surfels[x + 1]->getPosition() - surfels[x - 1]->getPosition();
		dY = 2.0f * (surfels[sizeX + x]->getPosition() - surfels[x]->getPosition());
		setRadiusAndNormal (surfels[x], dX, dY);
		
		// bottom row
		dX = surfels[(sizeY - 1) * sizeX + x + 1]->getPosition() - surfels[(sizeY - 1) * sizeX + x - 1]->getPosition();
		dY = 2.0f * (surfels[(sizeY - 1) * sizeX + x]->getPosition() - surfels[(sizeY - 2) * sizeX + x]->getPosition());
		setRadiusAndNormal (surfels[(sizeY - 1) * sizeX + x], dX, dY);

		// interior grid
		for (y = 1; y < sizeY - 1; y++) {
			dX = surfels[y * sizeX + x + 1]->getPosition() - surfels[y * sizeX + x - 1]->getPosition();
			dY = surfels[(y + 1) * sizeX + x]->getPosition() - surfels[(y - 1) * sizeX + x]->getPosition();
			setRadiusAndNormal (surfels[y * sizeX + x], dX, dY);
		}
	}


	for (y = 1; y < sizeY - 1; y++) {

		// left column
		dX = 2.0f * (surfels[y * sizeX + 1]->getPosition() - surfels[y * sizeX]->getPosition());
		dY = surfels[(y + 1) * sizeX]->getPosition() - surfels[(y - 1) * sizeX]->getPosition();			
		setRadiusAndNormal (surfels[y * sizeX], dX, dY);
		
		// right column
		dX = 2.0f * (surfels[y * sizeX + sizeX - 1]->getPosition() - surfels[y * sizeX + sizeX - 2]->getPosition());
		dY = surfels[(y + 2) * sizeX - 1]->getPosition() - surfels[y * sizeX - 1]->getPosition();
		setRadiusAndNormal (surfels[y * sizeX + sizeX - 1], dX, dY);
	}

	// corners
	// left top
	dX = 2.0f * (surfels[1]->getPosition() - surfels[0]->getPosition());
	dY = 2.0f * (surfels[sizeX]->getPosition() - surfels[0]->getPosition());
	setRadiusAndNormal (surfels[0], dX, dY);
	
	// right top
	dX = 2.0f * (surfels[sizeX - 1]->getPosition() - surfels[sizeX - 2]->getPosition());
	dY = 2.0f * (surfels[sizeX + sizeX - 1]->getPosition() - surfels[sizeX - 1]->getPosition());
	setRadiusAndNormal (surfels[sizeX - 1], dX, dY);
	
	// left bottom
	dX = 2.0f * (surfels[(sizeY - 1) * sizeX + 1]->getPosition() - surfels[(sizeY - 1) * sizeX]->getPosition());
	dY = 2.0f * (surfels[(sizeY - 1) * sizeX]->getPosition() - surfels[(sizeY - 2) * sizeX]->getPosition());
	setRadiusAndNormal (surfels[(sizeY - 1) * sizeX], dX, dY);
	
	// right bottom
	dX = 2.0f * (surfels[sizeY * sizeX - 1]->getPosition() - surfels[sizeY * sizeX - 2]->getPosition());
	dY = 2.0f * (surfels[sizeY * sizeX - 1]->getPosition() - surfels[(sizeY - 1) * sizeX - 1]->getPosition());
	setRadiusAndNormal (surfels[sizeY * sizeX - 1], dX, dY);
	
	
	// abort if surfel radius is too big
	if (surfelRadiusTooBig) {
		return false;
	}

	return true;

}

bool Painter::computeRadiiAndNormalsWithGaps (QArray<SurfelInterface *> surfels) {

	int x, y;
	Vector3D dX, dY;
	
	for (y = 0; y < sizeY; y++) {
		for (x = 0; x < sizeX; x++) {
			if(surfels[y * sizeX + x]) {
				if(computeDX(surfels, x, y, dX) && computeDY(surfels, x, y, dY)) {
					setRadiusAndNormal (surfels[y * sizeX + x], dX, dY);
				}
			}
		}
	}

	// TODO: check the surfel radii
	surfelRadiusTooBig = false;

	return true;
}

inline bool Painter::computeDX(QArray<SurfelInterface *> surfels, const int x, const int y, Vector3D& dX) {

	if( x > 0 && surfels[y*sizeX + x - 1] != 0 && x < sizeX-1 && surfels[y*sizeX + x + 1] != 0 ) {
		dX = surfels[y*sizeX + x + 1]->getPosition() - surfels[y*sizeX + x - 1]->getPosition();
		return true;
	}
	else if( x > 0 && surfels[y*sizeX + x - 1] != 0) {
		dX = 2.0f * (surfels[y*sizeX + x]->getPosition() - surfels[y*sizeX + x - 1]->getPosition());
		return true;
	}
	else if(x < sizeX-1 && surfels[y*sizeX + x + 1] != 0 ) {
		dX = 2.0f * (surfels[y*sizeX + x + 1]->getPosition() - surfels[y*sizeX + x]->getPosition());
		return true;
	}
	else {
		return false;
	}
}

inline bool Painter::computeDY(QArray<SurfelInterface *> surfels, const int x, const int y, Vector3D& dY) {
	if( y > 0 && surfels[(y-1)*sizeX + x] != 0 && y < sizeY-1 && surfels[(y+1)*sizeX + x] != 0 ) {
		dY = surfels[(y + 1) * sizeX + x]->getPosition() - surfels[(y - 1) * sizeX + x]->getPosition();
		return true;
	}
	else if( y > 0 && surfels[(y-1)*sizeX + x] != 0 ) {
		dY = 2.0f * (surfels[y * sizeX + x]->getPosition() - surfels[(y - 1) * sizeX + x]->getPosition());
		return true;
	}
	else if( y < sizeY-1 && surfels[(y+1)*sizeX + x] != 0 ) {
		dY = 2.0f * (surfels[(y+1) * sizeX + x]->getPosition() - surfels[y * sizeX + x]->getPosition());
		return true;
	}
	else {
		return false;
	}
}

inline void Painter::setRadiusAndNormal (SurfelInterface *surfel, Vector3D dX, Vector3D dY) {

	// surfel radius
	float radius = sqrt(MyMathTools::maxf (dX.getSquaredLength(), dY.getSquaredLength())) * 0.5f;
	if (radius > 4.0f* expectedRadius) {
		surfelRadiusTooBig = true;
	}

	scene->setSurfelRadius (surfel, radius, false);
				
	if (displacementChannel) {
		// surfel normal
		Vector3D newNormal = Vector3D::crossProduct (dY, dX);
		newNormal.normalize();
		scene->setSurfelNormal (surfel, newNormal, false);
	}
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
