//
// Title:   Painter.h
// Created: Mon Sep 23 13:40:18 2002
// Authors: Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __PAINTER_H_
#define __PAINTER_H_




#include "../../../DataTypes/src/MyDataTypes.h"
#include "../../../DataTypes/src/Vector3D.h"
#include "../../../Utilities/src/Matrix.h"
#include "../../Brush/src/Brush.h"
#include "../../OrthoMapper/src/OrthoMapper.h"
#include "../../DataStructures/src/SurfelCollection.h"
#include "../../Scene/src/Scene.h"
#include "../../Interfaces/src/FrameBufferInterface.h"


/**
 * Implements a painter. This class is used to perform surface editing operations using the brush tool.
 *
 * @author Mark Pauly
 * @version 1.2
 */

class Painter {


public:

	CORE_API Painter (const Brush *brush);
	CORE_API virtual ~Painter();

	
	/**
	 * Perform a paint operation.
	 *
	 * @param surfels
	 *        a list of all surfels that should be painted
	 * @param numSurfels
	 *        number of Surfels
	 * @param textureCoordinates
	 *        stores the texture coordinates for each surfel. If textureCoordinates = 0
	 *        the surfels are assumed to lie on a regular grid according to the brush resolution
	 * @param scale
	 *        scale for carving
	 * @param carveDirection
	 *        direction of carving
	 * @param maxDeviation
	 *        maximmum deviation of positions before and after displacement
	 * @param gaps
	 *        if the surfels lie on a regular grid and the grid contains gaps, i.e., positions, where
	 *        no surfel is stored (marked by a 0 pointer), this flag has to be set to true
	 *
     * @return returns true if painting was succesful
	*/
	CORE_API bool paint (QArray<SurfelInterface *> surfels, int numSurfels,
						 const QArray<MyDataTypes::TextureCoordinate> textureCoordinates,
						 const float scale = 1.0f, const Vector3D *carveDirection = 0,
						 const float maxDeviation = 2.0f, const bool gaps = false);

	/**
	 * Updates all relelant data structures when the brush has been changed.
	 *
	 * @param newBrush
	 *        pointer to the new brush
    */
	CORE_API void setBrush (const Brush *newBrush);



private:


	int	             sizeX,                 //  size of brush
		             sizeY; 
	
	BrushChannel     *displacementChannel,
                     *diffColorChannel,
					 *specColorChannel,
					 *ambCoeffChannel,
					 *diffCoeffChannel,
					 *specCoeffChannel,
					 *shininessChannel;

	ChannelComponent *displacementNormal,
		             *displacementCarving,
		             *displacementAlpha,		
	                 *diffRedComponent,
		             *diffGreenComponent,
		             *diffBlueComponent,
		             *diffColorAlpha,	
	                 *specRedComponent,
		             *specGreenComponent,
		             *specBlueComponent,
		             *specColorAlpha,	
					 *ambientCoefficient,
					 *ambientCoefficientAlpha,
	                 *diffuseCoefficient,
		             *diffuseCoefficientAlpha,	
	                 *specularCoefficient,
		             *specularCoefficientAlpha,
					 *shininess,
					 *shininessAlpha;

	Scene            *scene;		
	bool             surfelRadiusTooBig;
	float	         expectedRadius;    // expected average surfel radius

	float			 *filterBlendingGrid;

	bool computeRadiiAndNormals (QArray<SurfelInterface *> surfels);
	bool computeRadiiAndNormalsWithGaps (QArray<SurfelInterface *> surfels);

	inline void setRadiusAndNormal (SurfelInterface *surfel, Vector3D dx, Vector3D dy);

	inline bool computeDX(QArray<SurfelInterface *> surfels, const int x, const int y, Vector3D& dX);
	inline bool computeDY(QArray<SurfelInterface *> surfels, const int x, const int y, Vector3D& dY);


};


#endif  // __PAINTER_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
