//
// Title:   BaseSurfel.cpp
// Created: Mon Sep 23 14:03:58 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qcolor.h>

#include "../../DataTypes/src/Vector3D.h"
#include "BaseSurfel.h"

// **************
// static members
// **************

BaseSurfel *BaseSurfel::_memoryStart = 0;
uint BaseSurfel::SURFEL_ALLOCATION = 1024;

// TODO: update this property descriptor whenever changes are made to this class
const SurfelInterface::PropertyDescriptor BaseSurfel::PROPERTY_DESCRIPTOR = SurfelInterface::POSITION |
                                                                            SurfelInterface::NORMAL |
				                                                            SurfelInterface::RADIUS |
				                                                            SurfelInterface::DIFFUSE_COLOR |
															                SurfelInterface::SPECULAR_COLOR |
				                                                            SurfelInterface::FLAGS;


// **************
// public methods
// **************

Vector3D BaseSurfel::getPosition() const {
	return position;
}

Vector3D BaseSurfel::getNormal() const {
	return normal;
}

float BaseSurfel::getRadius() const {
	return radius;
}

QRgb BaseSurfel::getDiffuseColor() const {
	return diffuseColor;
}

QRgb BaseSurfel::getSpecularColor() const {
	return specularColor;
}

bool BaseSurfel::isFlagOn (const Flags flags) const {	
	return ((this->flags & flags) == flags);
}


bool BaseSurfel::isSelected () const {
	return this->flags & (SurfelInterface::SELECTED1 | SurfelInterface::SELECTED2 | SurfelInterface::SELECTED3);
}


SurfelInterface::Flags BaseSurfel::getFlags() const {
	return flags;
}


Vector3D BaseSurfel::getAxisOne() const {

	// TODO :not very efficient! both axis always computed
	Vector3D axisOne, axisTwo;
	getNormal().getTwoOrthogonals (axisOne, axisTwo);
	return radius * axisOne;
}

Vector3D BaseSurfel::getAxisTwo() const {
	
	// TODO :not very efficient! both axis always computed
	Vector3D axisOne, axisTwo;
	getNormal().getTwoOrthogonals (axisOne, axisTwo);
	return radius * axisTwo;
}

float BaseSurfel::getAmbientCoefficient() const {
	// return dummy value
	return 0.5f;
}

float BaseSurfel::getDiffuseCoefficient() const {
	// return dummy value
	return 0.75f;
}

float BaseSurfel::getSpecularCoefficient() const {
	// return dummy value
	return 0.25f;
}

float BaseSurfel::getShininess() const {
	// return dummy value
	return 0.75f;
}

MyDataTypes::TextureCoordinate BaseSurfel::getTextureCoordinate() const {
	// return dummy value
	MyDataTypes::TextureCoordinate textureCoordinate = {0.0f, 0.0f};
	return textureCoordinate;
}

SurfelInterface *BaseSurfel::copy() const {

	BaseSurfel *deepCopy;

	deepCopy = BaseSurfel::create (position, normal, radius, diffuseColor, specularColor, flags);
	return deepCopy;

}

// *****************
// protected methods
// *****************

BaseSurfel::BaseSurfel() {}

BaseSurfel::~BaseSurfel() {}

BaseSurfel* BaseSurfel::create (const Vector3D position, const Vector3D normal, const float radius,
						        const QRgb diffuseColor, const QRgb specularColor, const Flags flags) {

	BaseSurfel* newSurfel = new BaseSurfel();

	newSurfel->position      = position;
	newSurfel->normal        = normal;
	newSurfel->radius        = radius;
	newSurfel->diffuseColor  = diffuseColor;
	newSurfel->specularColor = specularColor;
	newSurfel->flags         = flags;

	return newSurfel;
}

void BaseSurfel::destroy() {
	this->_memoryNext = _memoryStart;
	_memoryStart = this;
}

void BaseSurfel::setPosition (const Vector3D newPosition) {
	position = newPosition;
}

void BaseSurfel::setNormal (const Vector3D newNormal) {
	normal = newNormal;
}

void BaseSurfel::setRadius (const float newRadius) {
	radius = newRadius;
}

void BaseSurfel::setDiffuseColor (const QRgb newDiffuseColor) {
	diffuseColor = newDiffuseColor;
}

void BaseSurfel::setSpecularColor (const QRgb newSpecularColor) {
	specularColor = newSpecularColor;
}

void BaseSurfel::setFlagOn (const Flags flags, const bool on) {
	
	if (on == true) {
		this->flags |= flags;
	}
	else {
		this->flags &= ~flags;
	}

}

void BaseSurfel::setFlags (const Flags newFlags) {
	flags = newFlags;
}

void BaseSurfel::setAxisOne (const Vector3D) {}
void BaseSurfel::setAxisTwo (const Vector3D) {}
void BaseSurfel::setAmbientCoefficient (const float) {}
void BaseSurfel::setDiffuseCoefficient (const float) {}
void BaseSurfel::setSpecularCoefficient (const float) {}
void BaseSurfel::setShininess (const float) {}
void BaseSurfel::setTextureCoordinate (const MyDataTypes::TextureCoordinate) {}

void BaseSurfel::copy (const SurfelInterface *sourceSurfel) {

	position = sourceSurfel->getPosition();
	normal   = sourceSurfel->getNormal();
	radius   = sourceSurfel->getRadius();
	diffuseColor = sourceSurfel->getDiffuseColor();
	specularColor = sourceSurfel->getSpecularColor();
	flags         = sourceSurfel->getFlags();

}

void* BaseSurfel::operator new (size_t) {

	BaseSurfel* surfel = _memoryStart;   
	if (surfel) 
		_memoryStart = _memoryStart->_memoryNext;
	else
	{
		// NOTE: the memory allocated for surfels never shrinks...
		surfel = (BaseSurfel *) new char [SURFEL_ALLOCATION * sizeof(BaseSurfel)];
		_memoryStart = &surfel[1];	// pointer to first FREE element!
		for (uint i = 0; i < SURFEL_ALLOCATION - 2; i++) {
			_memoryStart[i]._memoryNext = &_memoryStart[i+1];
		}
		_memoryStart[SURFEL_ALLOCATION - 2]._memoryNext = NULL;
    }

	return surfel;
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
