//
// Title:   ElSurfel.cpp
// Created: Mon Sep 23 14:03:36 2002
// Authors: Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qcolor.h>

#include "../../DataTypes/src/Vector3D.h"
#include "ElSurfel.h"

// **************
// static members
// **************

ElSurfel *ElSurfel::_memoryStart = 0;
uint ElSurfel::SURFEL_ALLOCATION = 1024;

// TODO: update this property descriptor whenever changes are made to this class
const SurfelInterface::PropertyDescriptor ElSurfel::PROPERTY_DESCRIPTOR = SurfelInterface::POSITION |
                                                                          SurfelInterface::NORMAL |
																		  SurfelInterface::TANGENT_AXES |
				                                                          SurfelInterface::DIFFUSE_COLOR |
				                                                          SurfelInterface::FLAGS;


// **************
// public methods
// **************

Vector3D ElSurfel::getPosition() const {
	return position;
}

Vector3D ElSurfel::getNormal() const {
	return normal;
}

Vector3D ElSurfel::getAxisOne() const {
	return axisOne;
}

Vector3D ElSurfel::getAxisTwo() const {
	return axisTwo;
}

QRgb ElSurfel::getDiffuseColor() const {
	return diffuseColor;
}


bool ElSurfel::isFlagOn (const Flags flags) const {	
	return ((this->flags & flags) == flags);
}


bool ElSurfel::isSelected () const {
	return this->flags & (SurfelInterface::SELECTED1 | SurfelInterface::SELECTED2 | SurfelInterface::SELECTED3);
}


SurfelInterface::Flags ElSurfel::getFlags() const {
	return flags;
}


float ElSurfel::getRadius() const {

	float len1 = axisOne.getLength();
	float len2 = axisTwo.getLength();
    return (len1 > len2) ? len1 : len2;
}

QRgb ElSurfel::getSpecularColor() const {
	// white
	return 0xffffffff;
}

float ElSurfel::getAmbientCoefficient() const {
	// return dummy value
	return 0.5f;
}

float ElSurfel::getDiffuseCoefficient() const {
	// return dummy value
	return 0.75f;
}

float ElSurfel::getSpecularCoefficient() const {
	// return dummy value
	return 0.25f;
}

float ElSurfel::getShininess() const {
	// return dummy value
	return 0.75f;
}

MyDataTypes::TextureCoordinate ElSurfel::getTextureCoordinate() const {
	// return dummy value
	MyDataTypes::TextureCoordinate textureCoordinate = {0.0f, 0.0f};
	return textureCoordinate;
}

SurfelInterface *ElSurfel::copy() const {

	ElSurfel *deepCopy;

	deepCopy = ElSurfel::create (position, normal, axisOne, axisTwo, diffuseColor, flags);
	return deepCopy;

}

// *****************
// protected methods
// *****************

ElSurfel::ElSurfel() {}

ElSurfel::~ElSurfel() {}

ElSurfel* ElSurfel::create (const Vector3D position, const Vector3D normal, const Vector3D axisOne,
							const Vector3D axisTwo, const QRgb diffuseColor, const Flags flags) {

	ElSurfel* newSurfel = new ElSurfel();

	newSurfel->position      = position;
	newSurfel->normal        = normal;
	newSurfel->axisOne         = axisOne;
	newSurfel->axisTwo         = axisTwo;
	newSurfel->diffuseColor  = diffuseColor;
	newSurfel->flags         = flags;

	return newSurfel;
}

void ElSurfel::destroy() {
	this->_memoryNext = _memoryStart;
	_memoryStart = this;
}

void ElSurfel::setPosition (const Vector3D newPosition) {
	position = newPosition;
}

void ElSurfel::setNormal (const Vector3D newNormal) {
	normal = newNormal;
}

void ElSurfel::setAxisOne (const Vector3D newAxisOne) {
	axisOne = newAxisOne;
}

void ElSurfel::setAxisTwo (const Vector3D newAxisTwo) {
	axisTwo = newAxisTwo;
}

void ElSurfel::setDiffuseColor (const QRgb newDiffuseColor) {
	diffuseColor = newDiffuseColor;
}


void ElSurfel::setFlagOn (const Flags flags, const bool on) {
	
	if (on == true) {
		this->flags |= flags;
	}
	else {
		this->flags &= ~flags;
	}

}

void ElSurfel::setFlags (const Flags newFlags) {
	flags = newFlags;
}

void ElSurfel::setRadius (const float newRadius) { (void)newRadius; }
void ElSurfel::setSpecularColor (const QRgb newSpecularColor) { (void)newSpecularColor; }
void ElSurfel::setAmbientCoefficient (const float) {}
void ElSurfel::setDiffuseCoefficient (const float) {}
void ElSurfel::setSpecularCoefficient (const float) {}
void ElSurfel::setShininess (const float) {}
void ElSurfel::setTextureCoordinate (const MyDataTypes::TextureCoordinate) {}

void ElSurfel::copy (const SurfelInterface *sourceSurfel) {

	position     = sourceSurfel->getPosition();
	normal       = sourceSurfel->getNormal();
	axisOne      = sourceSurfel->getAxisOne();
	axisTwo      = sourceSurfel->getAxisTwo();
	diffuseColor = sourceSurfel->getDiffuseColor();
	flags        = sourceSurfel->getFlags();

}

void* ElSurfel::operator new (size_t) {

	ElSurfel* surfel = _memoryStart;   
	if (surfel) 
		_memoryStart = _memoryStart->_memoryNext;
	else
	{
		// NOTE: the memory allocated for surfels never shrinks...
		surfel = (ElSurfel *) new char [SURFEL_ALLOCATION * sizeof(ElSurfel)];
		_memoryStart = &surfel[1];	// pointer to first FREE element!
		for (uint i = 0; i < SURFEL_ALLOCATION - 2; i++) {
			_memoryStart[i]._memoryNext = &_memoryStart[i+1];
		}
		_memoryStart[SURFEL_ALLOCATION - 2]._memoryNext = NULL;
    }

	return surfel;
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
