//
// Title:   ElSurfel.h
// Created: Mon Sep 23 14:03:28 2002
// Authors: Mark Pauly
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __ELSURFEL_H_
#define __ELSURFEL_H_

#include <qdatastream.h>
#include <qcolor.h>
#include <qlist.h>

#include "../../Core/Interfaces/src/SurfelInterface.h"
#include "Vector3D.h"
#include "DataTypesDLL.h"

/**
 * The class <code>ElSurfel</code> describes an elliptical surfel.
 * This <code>ElSurfel</code> supports the following attributes:
 * <ul>
 *   <li>Position      [<code>Vector3D</code>]</li>
 *   <li>Normal        [<code>Vector3D</code>]</li>
 *   <li>AxisOne       [<code>Vector3D</code>]</li>
 *   <li>AxisTwo       [<code>Vector3D</code>]</li>
 *   <li>Diffuse Color [<code>QRgb</code>]</li>
 *   <li>Flags         [<code>Flag</code>]</li>
 * </ul>
 *
 * <b>Note:</b> Surfels can only be created and <code>delete</code>d via the <code>SurfelCollection</code>
 *              class, which must always be in a consistent state
 * @author Oliver Knoll
 * @version 1.2
 */
class ElSurfel : public SurfelInterface {

public:

	/**
	 * The <code>PropertyDescriptor</code> of this <code>ElSurfel</code>.
	 */
	DATATYPES_API static const SurfelInterface::PropertyDescriptor PROPERTY_DESCRIPTOR;

	// ********************
	// supported properties
	// ********************

	DATATYPES_API Vector3D getPosition() const;
	DATATYPES_API Vector3D getNormal() const;
	DATATYPES_API Vector3D getAxisOne() const;
	DATATYPES_API Vector3D getAxisTwo() const;
	DATATYPES_API QRgb getDiffuseColor() const;
	DATATYPES_API bool isFlagOn (const Flags flags) const;
	DATATYPES_API bool isSelected () const;
	DATATYPES_API Flags getFlags() const;

	// **********************
	// unsupported properties
	// **********************

	DATATYPES_API virtual float getRadius() const;
	DATATYPES_API virtual QRgb getSpecularColor() const;
	DATATYPES_API virtual float getAmbientCoefficient() const;
	DATATYPES_API virtual float getDiffuseCoefficient() const;
	DATATYPES_API virtual float getSpecularCoefficient() const;
	DATATYPES_API virtual float getShininess() const;
	DATATYPES_API virtual MyDataTypes::TextureCoordinate getTextureCoordinate() const;

	DATATYPES_API virtual SurfelInterface *copy() const;

protected:

	// only the SurfelCollection can create surfels, as to make sure that the Objects consist of the same
	// surfel type
	friend class SurfelCollection;

	Vector3D position,
		     normal,
			 axisOne,
			 axisTwo;

	QRgb     diffuseColor;
	Flags    flags;  // 32 flags

	DATATYPES_API ElSurfel();
	DATATYPES_API virtual ~ElSurfel();

	/**
	 * Creates this <code>ElSurfel</code> with the given parameters.
	 */
	DATATYPES_API static ElSurfel* create (const Vector3D position  = Vector3D (0.0f, 0.0f, 0.0f),
		                                   const Vector3D normal    = Vector3D (0.0f, 0.0f, 1.0f),
		                                   const Vector3D axisOne   = Vector3D (1.0f, 0.0f, 0.0f),
		                                   const Vector3D axisTwo   = Vector3D (0.0f, 1.0f, 0.0f),
				                           const QRgb diffuseColor  = 0xffcccccc,   // some gray value
		                                   const Flags              = NO_FLAGS);

	// *************************
	// interface SurfelInterface
	// *************************

	DATATYPES_API virtual void destroy();

	// ********************
	// supported properties
	// ********************
	
	DATATYPES_API void setPosition (const Vector3D newPosition);
	DATATYPES_API void setNormal (const Vector3D newNormal);
	DATATYPES_API void setAxisOne (const Vector3D newAxisOne);
	DATATYPES_API void setAxisTwo (const Vector3D newAxisTwo);
	DATATYPES_API void setDiffuseColor (const QRgb newDiffuseColor);
	DATATYPES_API void setFlagOn (const Flags flags, const bool on);
	DATATYPES_API void setFlags (const Flags newFlags);
	
	// **********************
	// unsupported properties
	// **********************

	DATATYPES_API virtual void setRadius (const float newRadius);
	DATATYPES_API virtual void setSpecularColor (const QRgb newSpecularColor);
	DATATYPES_API virtual void setAmbientCoefficient (const float newAmbientCoefficient);
	DATATYPES_API virtual void setDiffuseCoefficient (const float newDiffuseCoefficient);
	DATATYPES_API virtual void setSpecularCoefficient (const float newSpecularCoefficient);
	DATATYPES_API virtual void setShininess (const float newShininess);
	DATATYPES_API virtual void setTextureCoordinate (const MyDataTypes::TextureCoordinate newTextureCoordinate);

	DATATYPES_API virtual void copy (const SurfelInterface *sourceSurfel);

private:

	// **************************
	// Internal memory management
	// **************************

	void* operator new (size_t);
	static uint SURFEL_ALLOCATION;
	static ElSurfel* _memoryStart;
    ElSurfel* _memoryNext;

};

#endif  // __ELSURFEL_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
