//
// Title:   Surfel1.h
// Created: Mon Sep 23 14:02:11 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SURFEL1_H_
#define __SURFEL1_H_

#include <qdatastream.h>
#include <qlist.h>

#include "MyDataTypes.h"
#include "BaseSurfel.h"
#include "DataTypesDLL.h"

/**
 * The class <code>Surfel1</code> subclasses the <code>BaseSurfel</code> and contains additionally the
 * following properties:
 * <ul>
 *   <li>diffuseCoefficient      [<code>float</code>]</li>
 *   <li>ambientCoefficient      [<code>float</code>]</li>
 *   <li>specularCoefficient     [<code>float</code>]</li>
 *   <li>shininess               [<code>float</code>]</li>
 *   <li>textureCoordinate      [<code>MyDataTypes::TextureCoordinate</code>]</li>
 * </ul>
 *
 * <b>Note:</b> Surfels can only be created and <code>delete</code>d via the <code>SurfelCollection</code>
 *              class, which must always be in a consistent state
 * @author Oliver Knoll, Mark Pauly, Matthias Zwicker
 * @version 1.2
 */
class Surfel1 : public BaseSurfel {

public:

	/**
	 * The defaul parameter for texture UV coordiantes with value (0.0, 0.0).
	 */
	DATATYPES_API static const MyDataTypes::TextureCoordinate defaultTextureCoordinate;

	/**
	 * The <code>PropertyDescriptor</code> of this <code>Surfel1</code>.
	 */
	DATATYPES_API static const SurfelInterface::PropertyDescriptor PROPERTY_DESCRIPTOR;

	// *************************
	// interface SurfelInterface
	// *************************

	// *********************************
	// additionally supported properties
	// *********************************

	DATATYPES_API float getAmbientCoefficient() const;
	DATATYPES_API float getDiffuseCoefficient() const;
	DATATYPES_API float getSpecularCoefficient() const;
	DATATYPES_API float getShininess() const;
	DATATYPES_API MyDataTypes::TextureCoordinate getTextureCoordinate() const;

	DATATYPES_API virtual SurfelInterface *copy() const;

protected:

	// only the SurfelCollection can create surfels, as to make sure that the Objects consist of the same
	// surfel type
	friend class SurfelCollection;

	float                          ambientCoefficient,
		                           diffuseCoefficient,
		                           specularCoefficient,
		                           shininess;
	MyDataTypes::TextureCoordinate textureCoordinate;

	DATATYPES_API Surfel1();
	DATATYPES_API virtual ~Surfel1();

	/**
	 * Creates this <code>Surfel1</code> with the given parameters.
	 */
	DATATYPES_API static Surfel1 *create (const Vector3D position                                = Vector3D (0.0f, 0.0f, 0.0f),
		                                  const Vector3D normal                                  = Vector3D (0.0f, 0.0f, 1.0f),
				                          const float radius                                     = 1.0f,
				                          const QRgb diffuseColor                                = 0xffcccccc,   // some gray value
						                  const QRgb specularColor                               = 0xffcccccc,   // some gray value
		                                  const Flags                                            = NO_FLAGS,
						                  const float ambientCoefficient                         = 0.5f,
						                  const float diffuseCoefficient                         = 0.75f,
						                  const float specularCoefficient                        = 0.25f,
						                  const float shininess                                  = 0.75f,
						                  const MyDataTypes::TextureCoordinate textureCoordinate = defaultTextureCoordinate);


	// *************************
	// interface SurfelInterface
	// *************************

	DATATYPES_API virtual void destroy();

	// *********************************
	// additionally supported properties
	// *********************************

	DATATYPES_API void setAmbientCoefficient (const float newAmbientCoefficient);
	DATATYPES_API void setDiffuseCoefficient (const float newDiffuseCoefficient);
	DATATYPES_API void setSpecularCoefficient (const float newSpecularCoefficient);
	DATATYPES_API void setShininess (const float newShnininess);
	DATATYPES_API void setTextureCoordinate (const MyDataTypes::TextureCoordinate newTextureCoordinate);

	DATATYPES_API virtual void copy (const SurfelInterface *sourceSurfel);

private:

	// **************************
	// Internal memory management
	// **************************

	void* operator new (size_t);
	static uint SURFEL_ALLOCATION;
	static Surfel1* _memoryStart;
	Surfel1* _memoryNext;

};


#endif  // __SURFEL1_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
