//
// Title:   api.h
// Created: Tue Sep 24 18:26:49 2002
// Authors: Tim Weyrich
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SFL_API_H__
#define __SFL_API_H__

// ### TODO
//
//   - Optional: Automatische Anwendung Transformations/Units/CoordSys
//     (Bisher nur CoordSys)
//
//   - Internen Fehlerzustand speichern -- Rueckgabe bei close() oder
//     getError(). (Evtl. auch getErrString() vorsehen.)
//
//   - Schalter fuer Fehlerlevel; Umleitungsmoeglichkeit fuer
//     Fehlerausgaben (z.B. qDebug()).
//

#include <pisystem.h>

#include "sfldefs.h"

#define  SFL_MAX_PROPERTY_FLAGS  8*32

namespace sfl
{
#include "normalenc.h"

  typedef enum
    {
      AMBIENT,
      DIFFUSE,
      SPECULAR,
      ANY                       // Doesn't work for coeffs!!
    }
  colorEnum;

  SFL_API void  memFree(void *mem);     //! Use this to free malloc'ed strings returned by the sfl-API.

  class SFL_API KillerSurfel
  {
    double  readTransform[32];

    double  positionTransform[16];
    bool    positionTransformIsId;
    bool    positionTransformIs3x3;
    bool    positionTransformIs3x4;

    double  normalTransform[16]; // Only preserving normalization, if normalTransformIs3x3 is true!! (or id, of course)
    bool    normalTransformIsId;
    bool    normalTransformIs3x3;
    bool    normalTransformIs3x4;

  public:
    typedef enum {
      /* enums must equal bit positions in SFLPROP_xxx flags! */

      POSITION_FIELD_IDX = 0,
      NORMAL_FIELD_IDX = 1,

      RADIUS_FIELD_IDX = 4,
      WEIGHT_FIELD_IDX = 5,
      SAMPLING_DENSITY_FIELD_IDX = 6,
      TEXTURE_UV_FIELD_IDX = 7,

      AMBIENT_COLOR_FIELD_IDX = 8,
      AMBIENT_COEFF_FIELD_IDX = 9,
      DIFFUSE_COLOR_FIELD_IDX = 10,
      DIFFUSE_COEFF_FIELD_IDX = 11,
      SPECULAR_COLOR_FIELD_IDX = 12,
      SPECULAR_COEFF_FIELD_IDX = 13,
      SHININESS_FIELD_IDX = 14,

      TANGENT_AXES_FIELD_IDX = 24,

      USER_FLAGS_FIELD_IDX = 27,
      NUM_FIELD_IDCS
    } FieldIdxEnum;

    uint32  surfelPadding; //!< Constantly 1 for now; will be read from tag SURFEL_SET_SURFEL_PADDING.

  private:
    uint32  propertyMask;
    uint32  propertyOffset[NUM_FIELD_IDCS];
    uint32  propertyPadding[NUM_FIELD_IDCS];
    uint32  propertySize[NUM_FIELD_IDCS];

    int16   layoutWriteUInt8ToUInt8Lookup[512][2]; // FIXME: Allocate array dynamically!
    int16   layoutWriteDoubleToFloatLookup[32][2]; // FIXME: Allocate array dynamically!

    uint32  propertyHints;

    typedef void  (KillerSurfel::*ConvFuncType)(void);
    ConvFuncType  conversions[NUM_FIELD_IDCS];
    int   numConversions;

    uint32  colorSize;  //!< Disk size of color information, according to the current color model.
    uint32  diskSize;           //!< Disk size of one surfel on disk, including surfel padding.

    void  updatePropertyLayout(int diskSizeOverride=-1, int numProps=-1, const uint16 *propIndcs=NULL, const uint32 *propOffs=NULL);
    void  updateReadConversions(void);
    void  updateWriteConversions(void);

    void  convNormalToIco8(void);
    void  convNormalToCub16(void);
    void  convNormalToCub32(void);

    void  convIco8ToNormal(void);
    void  convCub16ToNormal(void);
    void  convCub32ToNormal(void);
    void  convRadiusToSamplingDensity(void);
    void  convRadiusToSamplingDensityWithoutNormal(void);
    void  convSamplingDensityToRadius(void);
    void  convColorModelRGBToL(void);
    void  convColorModelLToRGB(void);
    void  convColorModelRGBToYUV(void);
    void  convColorModelYUVToRGB(void);
    void  convColorModelRGBToYIQ(void);
    void  convColorModelYIQToRGB(void);
    void  convColorModelYUVToYIQ(void);
    void  convColorModelYIQToYUV(void);
    void  convCopyColorAmbientToAny(void);
    void  convCopyColorDiffuseToAny(void);
    void  convCopyColorSpecularToAny(void);

    void  convRadiusNormalToTangentAxes(void);
    void  convTangentAxesToRadius(void);
    void  convTangentAxesToNormal(void);

    void  convPositionTransform3x3(void);
    void  convPositionTransform3x4(void);
    void  convPositionTransform(void);
    void  convNormalTransform3x3(void);
    void  convNormalTransform3x4(void);
    void  convNormalTransform(void);

    uint32  spillGeneric(uint8 *dest) const;
    uint32  slurpGeneric(uint8 *src) const;

  public:
    KillerSurfel();

    double         position[3];
    double         normal[3];
    union {
      uint8   ico8;
      uint16  cub16;
      uint32  cub32;
    } normalCmpr; //!< Compressed normal.
    double         radius;
    double         weight;
    double         samplingDensity[3];
    double         textureUV[2];
    unsigned char  colors[4][4]; //!< RGBA, LxxA, or YUVA for Amb./Diff./Spec./Any.
    unsigned char  coeffs[4];   //!< Coefficients for Amb./Diff./Spec./Any.
    double         shininess;

    double         tangentAxes[2][3];

    unsigned       userFlags;
#if 0
    void          *userFields[8];
    unsigned       userTypes[8];
#endif

    KillerSurfel  &setPropertyMask(uint32 mask);
    KillerSurfel  &setPropertyMask(uint32 mask, int diskSizeOverride, int numProps, const uint16 *propIndcs, const uint32 *propOffs);
    uint32         getPropertyMask(void) const { return propertyMask; }

    KillerSurfel  &setPropertyHints(uint32 hints);
    uint32         getPropertyHints(void) const { return propertyHints; }

    KillerSurfel  &setReadTransform(double readTransform[32]);

    uint32  getDiskSize(void) const { return diskSize; }

    uint32  getPropertyOffset(FieldIdxEnum idx) const { return propertyOffset[(int)idx]; }
    uint32  getPropertyPadding(FieldIdxEnum idx) const { return propertyPadding[(int)idx]; }
    uint32  getPropertySize(FieldIdxEnum idx) const { return propertySize[(int)idx]; }

    PI_INLINE normalenc_compression_enum  getNormalEncoding(void) const
    { return (normalenc_compression_enum)((propertyMask & SFLPROP_NORMAL_COMPR_MASK) >> SFLPROP_NORMAL_COMPR_SHIFT); }

    uint32  (KillerSurfel::*spill)(uint8 *dest) const;
    uint32  (KillerSurfel::*slurp)(uint8 *src) const;

    PI_INLINE void  applyConversions(void)
    {
      int  i;
      for (i=0; i<numConversions; i++)
        (this->*conversions[i])();
    }
  };

  class SFL_API Stream
  {
  protected:
    gff_file       *gff;

    KillerSurfel  surfelBuffer;

    size_t        rwBufferSize;
    uint8        *rwBuffer[2];  //!< Two buffers of size rwBufferSize. rwBuffer[1] follows \e directly rwBuffer[0].
    uint8        *rwBufferHead; //!< Buffer write position.
    uint8        *rwBufferTail; //!< Buffer read position.

    uint32  fillRwBuffer(void); //!< Reads at max rwBufferSize bytes from the gff stream.
    uint32  flushRwBuffer0(void); //!< Writes rwBuffer[0] to the gff stream.
    uint32  flushRwBuffer(void); //!< Writes both buffers to the gff stream.

    bool  apiRowMajorMatrices;

    void  apiSwapMat4x4(float mat[16]) const;
    void  apiSwapMat4x4(double mat[16]) const;
    void  apiSwapMat4x4ToTemp4x4(float temp[16], const float mat[16]) const;
    void  apiSwapMat4x4ToTemp4x4(double temp[16], const double mat[16]) const;

    bool  apiGetTransform4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetTransform4x4(const double *src, double *mat, bool internal=false) const;
    bool  apiGetInvTransform4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetInvTransform4x4(const double *src, double *mat, bool internal=false) const;
    bool  apiGetRotation4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetRotation4x4(const double *src, double *mat, bool internal=false) const;
    bool  apiGetScalationRotation4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetScalationRotation4x4(const double *src, double *mat, bool internal=false) const;
    bool  apiGetTranslation4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetTranslation4x4(const double *src, double *mat, bool internal=false) const;
    bool  apiGetScalationTranslation4x4(const double *src, float *mat, bool internal=false) const;
    bool  apiGetScalationTranslation4x4(const double *src, double *mat, bool internal=false) const;

    Stream();
    ~Stream();

  public:
    void  setMatrixApiRowMajor(void) { apiRowMajorMatrices = true; }
    void  setMatrixApiColumnMajor(void) { apiRowMajorMatrices = false; }
  };

  class SFL_API InStream : public sfl::Stream
  {
  private:
    gff_directory  *header;
    double          defaultUnits;
    double          fileCoordSystem[32];
    double          appCoordSystem[32];

    double          sceneCameraTransform[32];

    bool            readWorldFlag;

    gff_directory  *surfelSetList;

    gff_directory  *surfelSet;
    int             surfelSetIdx;
    uint32          surfelSetFormatVersion;
    uint32          surfelSetNumResolutions;
    uint32          surfelSetProperties[8];
    double          surfelSetUnits;
    double          surfelSetTransform[32];
    int             surfelSetNumMarkers;

    gff_directory  *resolution;
    int             resolutionIdx;
    uint32          resolutionNumSurfels;

    uint32  hdVersion;

    static bool    queryField(gff_directory *dir, int tagId) { return !gff_query_field(dir, tagId, NULL, NULL); }
    static int     queryFieldNumElements(gff_directory *dir, int tagId)
    { int n; if (!gff_query_field(dir, tagId, NULL, &n)) return n; else return -1; }

    static char   *getFieldString(gff_directory *dir, int tagId, const char *deflt=NULL);
    static bool    getFieldString(gff_directory *dir, int tagId, char *dest, size_t buflen, const char *deflt=NULL);
    static bool    getFieldUInt8(gff_directory *dir, int tagId, uint8 &val, bool haveDefault=false, uint8 deflt=0);
    static uint8   getFieldUInt8(gff_directory *dir, int tagId, bool haveDefault=false, uint8 deflt=0);
    static bool    getFieldUInt32(gff_directory *dir, int tagId, uint32 &val, bool haveDefault=false, uint32 deflt=0);
    static uint32  getFieldUInt32(gff_directory *dir, int tagId, bool haveDefault=false, uint32 deflt=0);
    static bool    getFieldDouble(gff_directory *dir, int tagId, double &val, bool haveDefault=false, double deflt=0.0);
    static double  getFieldDouble(gff_directory *dir, int tagId, bool haveDefault=false, double deflt=0.0);

    static uint32  getFieldUInt32ArrIdx(gff_directory *dir, int tagId, int idx, uint32 deflt);
    static double  getFieldDoubleArrIdx(gff_directory *dir, int tagId, int idx, double deflt);

    InStream();
    ~InStream();

  public:
    static InStream  *open(const char *fName, const char *mode="");
    static int        close(InStream *in);

    // Query methods return true, if the corresponding field is defined.
    //

    // Header Field Queries
    bool   queryTitle(void) const       { return queryField(header, SFLTAG_TITLE); }
    bool   queryAuthor(void) const      { return queryField(header, SFLTAG_AUTHOR); }
    bool   queryApplication(void) const { return queryField(header, SFLTAG_APPLICATION); }
    bool   queryDescription(void) const { return queryField(header, SFLTAG_DESCRIPTION); }
    bool   queryVersion(void) const     { return queryField(header, SFLTAG_VERSION); }
    bool   queryDate(void) const        { return queryField(header, SFLTAG_DATE); }

    bool   queryRightVector(void) const { return queryField(header, SFLTAG_RIGHT_VEC); }
    bool   queryUpVector(void) const    { return queryField(header, SFLTAG_UP_VEC); }
    bool   queryBackVector(void) const  { return queryField(header, SFLTAG_BACK_VEC); }

    bool   querySceneCameraTransform(void) const { return queryField(header, SFLTAG_SCENE_CAMERA_TRANSFORM); }

    bool   queryDefaultUnits(void) const{ return queryField(header, SFLTAG_DEFAULT_UNITS); }
    bool   queryOverAllBoundingBox(void) const { return queryField(header, SFLTAG_BOUNDING_BOX); }

    // Get methods returning a boolean return \c true, if the
    // corresponding field is defined. (If a default value exists,
    // they \e always return \c true.)
    //
    // The get methods are not coded very efficiently.
    //

    // Header Field Gets
    char  *getTitle(void) const                         { return getFieldString(header, SFLTAG_TITLE); }
    bool   getTitle(char *dest, size_t buflen) const    { return getFieldString(header, SFLTAG_TITLE, dest, buflen); }
    char  *getAuthor(void) const                        { return getFieldString(header, SFLTAG_AUTHOR); }
    bool   getAuthor(char *dest, size_t buflen) const   { return getFieldString(header, SFLTAG_AUTHOR, dest, buflen); }
    char  *getApplication(void) const                   { return getFieldString(header, SFLTAG_APPLICATION); }
    bool   getApplication(char *dest, size_t buflen) const { return getFieldString(header, SFLTAG_APPLICATION, dest, buflen); }
    char  *getDescription(void) const                   { return getFieldString(header, SFLTAG_DESCRIPTION); }
    bool   getDescription(char *dest, size_t buflen) const {return getFieldString(header, SFLTAG_DESCRIPTION, dest, buflen); }
    bool    getVersion(uint32 &version) const           { return getFieldUInt32(header, SFLTAG_VERSION, version); }
    uint32  getVersion(void) const                      { return getFieldUInt32(header, SFLTAG_VERSION); }
    char   *getDate(void) const                         { return getFieldString(header, SFLTAG_DATE); }
    bool    getDate(char *dest, size_t buflen) const    { return getFieldString(header, SFLTAG_DATE, dest, buflen); }

    bool    getRightVector(double &x, double &y, double &z) const
    {
      x = fileCoordSystem[0];
      y = fileCoordSystem[4];
      z = fileCoordSystem[8];
      return true;
    }
    bool    getUpVector(double &x, double &y, double &z) const
    {
      x = fileCoordSystem[1];
      y = fileCoordSystem[5];
      z = fileCoordSystem[9];
      return true;
    }
    bool    getBackVector(double &x, double &y, double &z) const
    {
      x = fileCoordSystem[2];
      y = fileCoordSystem[6];
      z = fileCoordSystem[10];
      return true;
    }

    bool  getSceneCameraTransform4x4(float *mat) const { return apiGetTransform4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraTransform4x4(double *mat) const { return apiGetTransform4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraInvTransform4x4(float *mat) const { return apiGetInvTransform4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraInvTransform4x4(double *mat) const { return apiGetInvTransform4x4(sceneCameraTransform, mat); }

    bool  getSceneCameraRotation4x4(float *mat) const { return apiGetRotation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraRotation4x4(double *mat) const { return apiGetRotation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraScalationRotation4x4(float *mat) const { return apiGetScalationRotation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraScalationRotation4x4(double *mat) const { return apiGetScalationRotation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraTranslation4x4(float *mat) const { return apiGetTranslation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraTranslation4x4(double *mat) const { return apiGetTranslation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraScalationTranslation4x4(float *mat) const { return apiGetScalationTranslation4x4(sceneCameraTransform, mat); }
    bool  getSceneCameraScalationTranslation4x4(double *mat) const { return apiGetScalationTranslation4x4(sceneCameraTransform, mat); }

    bool    getDefaultUnits(double &units) const        { units = defaultUnits; return true; }
    double  getDefaultUnits(void) const                 { return defaultUnits; }

    bool   getOverAllBoundingBox(double &x1, double &y1, double &z1, double &x2, double &y2, double &z2)
    {
      x1 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 0, 0.0);
      y1 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 1, 0.0);
      z1 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 2, 0.0);
      x2 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 3, 0.0);
      y2 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 4, 0.0);
      z2 = getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 5, 0.0);
      return queryOverAllBoundingBox();
    }

    bool   getOverAllBoundingBox(float &x1, float &y1, float &z1, float &x2, float &y2, float &z2)
    {
      x1 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 0, 0.0);
      y1 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 1, 0.0);
      z1 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 2, 0.0);
      x2 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 3, 0.0);
      y2 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 4, 0.0);
      z2 = (float)getFieldDoubleArrIdx(header, SFLTAG_BOUNDING_BOX, 5, 0.0);
      return queryOverAllBoundingBox();
    }

    // Surfel Set List Field Gets
    bool    getNumSurfelSets(uint32 &numSets) const  { return getFieldUInt32(surfelSetList, SFLTAG_NUM_SURFEL_SETS, numSets, true, 0); }
    uint32  getNumSurfelSets(void) const             { return getFieldUInt32(surfelSetList, SFLTAG_NUM_SURFEL_SETS, true, 0); }

    // Seeking A Single Surfel Set
    bool  seekSurfelSet(int idx);

    // Surfel Set Queries/Gets
    uint32  getSurfelSetFormatVersion(void) const { return surfelSetFormatVersion; }
    uint32  getSurfelSetProperties(int idx) const { return (surfelSetIdx >= 0 && idx >= 0 && idx < SFL_MAX_PROPERTY_FLAGS/32
                                                            ? surfelSetProperties[idx]
                                                            : 0); }

    bool  querySurfelSetRes0Index(void) const   { return queryField(surfelSet, SFLTAG_SURFEL_SET_RES0_IDX); }
    bool  querySurfelSetUnits(void) const       { return queryField(surfelSet, SFLTAG_SURFEL_SET_UNITS); }
    bool  querySurfelSetTransform(void) const   { return queryField(surfelSet, SFLTAG_SURFEL_SET_TRANSFORM); }
    bool  querySurfelSetBoundingBox(void) const { return queryField(surfelSet, SFLTAG_BOUNDING_BOX); }

    bool   getSurfelSetRes0Index(int &idx) const { idx = (int)getFieldUInt32(surfelSet,
                                                                             SFLTAG_SURFEL_SET_RES0_IDX,
                                                                             true, surfelSetNumResolutions-1); return true; }
    int    getSurfelSetRes0Index(void) const     { return (int)getFieldUInt32(surfelSet,
                                                                              SFLTAG_SURFEL_SET_RES0_IDX,
                                                                              true, surfelSetNumResolutions-1); }

    bool    getSurfelSetUnits(double &units) const      { units = surfelSetUnits; return true; }
    double  getSurfelSetUnits(void) const               { return surfelSetUnits; }

    bool  getSurfelSetTransform4x4(float *mat) const { return apiGetTransform4x4(surfelSetTransform, mat); }
    bool  getSurfelSetTransform4x4(double *mat) const { return apiGetTransform4x4(surfelSetTransform, mat); }
    bool  getSurfelSetInvTransform4x4(float *mat) const { return apiGetInvTransform4x4(surfelSetTransform, mat); }
    bool  getSurfelSetInvTransform4x4(double *mat) const { return apiGetInvTransform4x4(surfelSetTransform, mat); }

    bool  getSurfelSetRotation4x4(float *mat) const { return apiGetRotation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetRotation4x4(double *mat) const { return apiGetRotation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetScalationRotation4x4(float *mat) const { return apiGetScalationRotation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetScalationRotation4x4(double *mat) const { return apiGetScalationRotation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetTranslation4x4(float *mat) const { return apiGetTranslation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetTranslation4x4(double *mat) const { return apiGetTranslation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetScalationTranslation4x4(float *mat) const { return apiGetScalationTranslation4x4(surfelSetTransform, mat); }
    bool  getSurfelSetScalationTranslation4x4(double *mat) const { return apiGetScalationTranslation4x4(surfelSetTransform, mat); }
                                //
    bool   getSurfelSetBoundingBox(double &x1, double &y1, double &z1, double &x2, double &y2, double &z2)
    {
      x1 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 0, 0.0);
      y1 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 1, 0.0);
      z1 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 2, 0.0);
      x2 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 3, 0.0);
      y2 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 4, 0.0);
      z2 = getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 5, 0.0);
      return querySurfelSetBoundingBox();
    }

    bool   getSurfelSetBoundingBox(float &x1, float &y1, float &z1, float &x2, float &y2, float &z2)
    {
      x1 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 0, 0.0);
      y1 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 1, 0.0);
      z1 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 2, 0.0);
      x2 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 3, 0.0);
      y2 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 4, 0.0);
      z2 = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_BOUNDING_BOX, 5, 0.0);
      return querySurfelSetBoundingBox();
    }

    char  *getSurfelSetIdentifier(void) const;
    bool   getSurfelSetIdentifier(char *dest, size_t buflen) const;

    bool    getSurfelSetNumMarkers(int &numMarkers) const { numMarkers = surfelSetNumMarkers; return true; }
    int     getSurfelSetNumMarkers(void) const            { return surfelSetNumMarkers; }

    bool    getSurfelSetMarkerSurfelIndex(int markerIdx, uint32 &surfelIdx) const
    { surfelIdx = getFieldUInt32ArrIdx(surfelSet, SFLTAG_SURFEL_SET_MARKER_IDXVEC, markerIdx, 0); return true; }
    bool    getSurfelSetMarkerU(int markerIdx, float &u) const
    { u = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_SURFEL_SET_MARKER_UVEC, markerIdx, 0.0); return true; }
    double  getSurfelSetMarkerU(int markerIdx) const
    { return getFieldDoubleArrIdx(surfelSet, SFLTAG_SURFEL_SET_MARKER_UVEC, markerIdx, 0.0); }
    bool    getSurfelSetMarkerV(int markerIdx, float &v) const
    { v = (float)getFieldDoubleArrIdx(surfelSet, SFLTAG_SURFEL_SET_MARKER_VVEC, markerIdx, 0.0); return true; }
    double  getSurfelSetMarkerV(int markerIdx) const
    { return getFieldDoubleArrIdx(surfelSet, SFLTAG_SURFEL_SET_MARKER_VVEC, markerIdx, 0.0); }

    // What coordinate system interpretation does the reader have?
    InStream  &setLocalRightVector(double x, double y, double z);
    InStream  &setLocalUpVector(double x, double y, double z);
    InStream  &setLocalBackVector(double x, double y, double z);

    //! Set coordinate system for reads.
    /*! If called \e before seekSurfelSet(), subsequent calls to
     *  readPosition()/readNormal() yield coordinates in world
     *  coordinates, or object coordinates, respectively, according to
     *  \a readWorld.
     *
     *  For a newly created stream, readWorld is \c false by
     *  default. Changing readAsWorldCoordinates() affects all
     *  subsequent surfel sets until changed again by
     *  setReadAsWorldCoordinates().
     *
     *    \param        readWorld       \c true, if world coordinates shall be read.
     */
    InStream  &setReadAsWorldCoordinates(bool readWorld) { readWorldFlag = readWorld; return *this; }

    // Announcing Properties, that may be read.
    void  setSurfelSetPropertyHints(uint32 propMask=0xffefffffUL) { surfelBuffer.setPropertyHints(propMask); }

    // Seeking A Single Resolution
    bool  seekResolution(int idx);

    uint32  getResolutionNumSurfels(void) const { return resolutionNumSurfels; }

    PI_INLINE int  beginSurfel(void)
    {
      if (rwBufferTail + surfelBuffer.getDiskSize() > rwBufferHead)
        if (fillRwBuffer() < surfelBuffer.getDiskSize())
          return 1;
      rwBufferTail += (surfelBuffer.*surfelBuffer.slurp)(rwBufferTail);
      surfelBuffer.applyConversions();
      return 0;
    }
    PI_INLINE int  endSurfel(void) { return 0; }

    PI_INLINE void  readSurfelPosition3(double &x, double &y, double &z)
    {
      x = surfelBuffer.position[0];
      y = surfelBuffer.position[1];
      z = surfelBuffer.position[2];
    }

    PI_INLINE void  readSurfelPosition3(float &x, float &y, float &z)
    {
      x = (float)surfelBuffer.position[0];
      y = (float)surfelBuffer.position[1];
      z = (float)surfelBuffer.position[2];
    }

    PI_INLINE void  readSurfelPosition4(double &x, double &y, double &z, double &w)
    {
      x = surfelBuffer.position[0];
      y = surfelBuffer.position[1];
      z = surfelBuffer.position[2];
      w = 1.0;
    }

    PI_INLINE void  readSurfelPosition4(float &x, float &y, float &z, float &w)
    {
      x = (float)surfelBuffer.position[0];
      y = (float)surfelBuffer.position[1];
      z = (float)surfelBuffer.position[2];
      w = 1.0f;
    }

    PI_INLINE void  readSurfelNormal3(double &x, double &y, double &z)
    {
      x = surfelBuffer.normal[0];
      y = surfelBuffer.normal[1];
      z = surfelBuffer.normal[2];
    }

    PI_INLINE void  readSurfelNormal3(float &x, float &y, float &z)
    {
      x = (float)surfelBuffer.normal[0];
      y = (float)surfelBuffer.normal[1];
      z = (float)surfelBuffer.normal[2];
    }

    PI_INLINE void  readSurfelNormal4(double &x, double &y, double &z, double &w)
    {
      x = surfelBuffer.normal[0];
      y = surfelBuffer.normal[1];
      z = surfelBuffer.normal[2];
      w = 0.0;
    }

    PI_INLINE void  readSurfelNormal4(float &x, float &y, float &z, float &w)
    {
      x = (float)surfelBuffer.normal[0];
      y = (float)surfelBuffer.normal[1];
      z = (float)surfelBuffer.normal[2];
      w = 0.0f;
    }

    PI_INLINE double  readSurfelRadius(void) { return surfelBuffer.radius; }
    PI_INLINE void    readSurfelRadius(double &r) { r = surfelBuffer.radius; }
    PI_INLINE void    readSurfelRadius(float &r) { r = (float)surfelBuffer.radius; }

    PI_INLINE double  readSurfelWeight(void) { return surfelBuffer.weight; }
    PI_INLINE void    readSurfelWeight(double &w) { w = surfelBuffer.weight; }
    PI_INLINE void    readSurfelWeight(float &w) { w = (float)surfelBuffer.weight; }

    PI_INLINE void  readSurfelSamplingDensity3(double &x, double &y, double &z)
    {
      x = surfelBuffer.samplingDensity[0];
      y = surfelBuffer.samplingDensity[1];
      z = surfelBuffer.samplingDensity[2];
    }

    PI_INLINE void  readSurfelSamplingDensity3(float &x, float &y, float &z)
    {
      x = (float)surfelBuffer.samplingDensity[0];
      y = (float)surfelBuffer.samplingDensity[1];
      z = (float)surfelBuffer.samplingDensity[2];
    }

    PI_INLINE void  readSurfelSamplingDensity4(double &x, double &y, double &z, double &w)
    {
      x = surfelBuffer.samplingDensity[0];
      y = surfelBuffer.samplingDensity[1];
      z = surfelBuffer.samplingDensity[2];
      w = 1.0;
    }

    PI_INLINE void  readSurfelSamplingDensity4(float &x, float &y, float &z, float &w)
    {
      x = (float)surfelBuffer.samplingDensity[0];
      y = (float)surfelBuffer.samplingDensity[1];
      z = (float)surfelBuffer.samplingDensity[2];
      w = 1.0f;
    }

    PI_INLINE void  readSurfelTextureUV(float &u, float &v)
    {
      u = (float)surfelBuffer.textureUV[0];
      v = (float)surfelBuffer.textureUV[1];
    }

    PI_INLINE double  readSurfelColorRf(colorEnum colIdx)
    {
      return (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorRf(colorEnum colIdx, double &r)
    {
      r = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorRf(colorEnum colIdx, float &r)
    {
      r = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][0]);
    }

    PI_INLINE double  readSurfelColorGf(colorEnum colIdx)
    {
      return (1.0/255.0) * surfelBuffer.colors[(int)colIdx][1];
    }

    PI_INLINE void  readSurfelColorGf(colorEnum colIdx, double &g)
    {
      g = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][1];
    }

    PI_INLINE void  readSurfelColorGf(colorEnum colIdx, float &g)
    {
      g = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][1]);
    }

    PI_INLINE double  readSurfelColorBf(colorEnum colIdx)
    {
      return (1.0/255.0) * surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE void  readSurfelColorBf(colorEnum colIdx, double &b)
    {
      b = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE void  readSurfelColorBf(colorEnum colIdx, float &b)
    {
      b = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][2]);
    }

    PI_INLINE double  readSurfelColorAf(colorEnum colIdx)
    {
      return (1.0/255.0) * surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorAf(colorEnum colIdx, double &a)
    {
      a = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorAf(colorEnum colIdx, float &a)
    {
      a = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][3]);
    }

    PI_INLINE double  readSurfelColorLf(colorEnum colIdx)
    {
      return (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorLf(colorEnum colIdx, double &l)
    {
      l = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorLf(colorEnum colIdx, float &l)
    {
      l = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][0]);
    }

    PI_INLINE void  readSurfelColorLAf(colorEnum colIdx, double &l, double &a)
    {
      l = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
      a = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorLAf(colorEnum colIdx, float &l, float &a)
    {
      l = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][0]);
      a = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][3]);
    }

    PI_INLINE void  readSurfelColorRGBf(colorEnum colIdx, double &r, double &g, double &b)
    {
      r = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
      g = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][1];
      b = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE void  readSurfelColorRGBf(colorEnum colIdx, float &r, float &g, float &b)
    {
      r = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][0]);
      g = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][1]);
      b = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][2]);
    }

    PI_INLINE void  readSurfelColorRGBAf(colorEnum colIdx, double &r, double &g, double &b, double &a)
    {
      r = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][0];
      g = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][1];
      b = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][2];
      a = (1.0/255.0) * surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorRGBAf(colorEnum colIdx, float &r, float &g, float &b, float &a)
    {
      r = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][0]);
      g = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][1]);
      b = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][2]);
      a = (float)((1.0/255.0) * surfelBuffer.colors[(int)colIdx][3]);
    }

#   define  readSurfelColorYf     readSurfelColorRf
#   define  readSurfelColorUf     readSurfelColorGf
#   define  readSurfelColorVf     readSurfelColorBf
#   define  readSurfelColorYUVf   readSurfelColorRGBf
#   define  readSurfelColorYUVAf  readSurfelColorRGBAf

    PI_INLINE unsigned char  readSurfelColorRuc(colorEnum colIdx)
    {
      return surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorRuc(colorEnum colIdx, unsigned char &r)
    {
      r = surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE unsigned char  readSurfelColorGuc(colorEnum colIdx)
    {
      return surfelBuffer.colors[(int)colIdx][1];
    }

    PI_INLINE void  readSurfelColorGuc(colorEnum colIdx, unsigned char &g)
    {
      g = surfelBuffer.colors[(int)colIdx][1];
    }

    PI_INLINE unsigned char  readSurfelColorBuc(colorEnum colIdx)
    {
      return surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE void  readSurfelColorBuc(colorEnum colIdx, unsigned char &b)
    {
      b = surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE unsigned char  readSurfelColorAuc(colorEnum colIdx)
    {
      return surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorAuc(colorEnum colIdx, unsigned char &a)
    {
      a = surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE unsigned char  readSurfelColorLuc(colorEnum colIdx)
    {
      return surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorLuc(colorEnum colIdx, unsigned char &l)
    {
      l = surfelBuffer.colors[(int)colIdx][0];
    }

    PI_INLINE void  readSurfelColorLAuc(colorEnum colIdx, unsigned char &l, unsigned char &a)
    {
      l = surfelBuffer.colors[(int)colIdx][0];
      a = surfelBuffer.colors[(int)colIdx][3];
    }

    PI_INLINE void  readSurfelColorRGBuc(colorEnum colIdx, unsigned char &r, unsigned char &g, unsigned char &b)
    {
      r = surfelBuffer.colors[(int)colIdx][0];
      g = surfelBuffer.colors[(int)colIdx][1];
      b = surfelBuffer.colors[(int)colIdx][2];
    }

    PI_INLINE void  readSurfelColorRGBAuc(colorEnum colIdx, unsigned char &r, unsigned char &g, unsigned char &b, unsigned char &a)
    {
      r = surfelBuffer.colors[(int)colIdx][0];
      g = surfelBuffer.colors[(int)colIdx][1];
      b = surfelBuffer.colors[(int)colIdx][2];
      a = surfelBuffer.colors[(int)colIdx][3];
    }

#   define  readSurfelColorYuc     readSurfelColorRuc
#   define  readSurfelColorUuc     readSurfelColorGuc
#   define  readSurfelColorVuc     readSurfelColorBuc
#   define  readSurfelColorYUVuc   readSurfelColorRGBuc
#   define  readSurfelColorYUVAuc  readSurfelColorRGBAuc

    PI_INLINE double  readSurfelColorCoefff(colorEnum colIdx)
    {
      // FIXME: Is that Tisu's semantic?
      return (1.0/255.0) * surfelBuffer.coeffs[(int)colIdx];
    }

    PI_INLINE void  readSurfelColorCoefff(colorEnum colIdx, double &c)
    {
      // FIXME: Is that Tisu's semantic?
      c = (1.0/255.0) * surfelBuffer.coeffs[(int)colIdx];
    }

    PI_INLINE void  readSurfelColorCoefff(colorEnum colIdx, float &c)
    {
      // FIXME: Is that Tisu's semantic?
      c = (float)((1.0/255.0) * surfelBuffer.coeffs[(int)colIdx]);
    }

    PI_INLINE unsigned char  readSurfelColorCoeffuc(colorEnum colIdx)
    {
      return surfelBuffer.coeffs[(int)colIdx];
    }

    PI_INLINE void  readSurfelColorCoeffuc(colorEnum colIdx, unsigned char &c)
    {
      c = surfelBuffer.coeffs[(int)colIdx];
    }

    PI_INLINE void  readSurfelShininess(double &s) { s = surfelBuffer.shininess; }
    PI_INLINE void  readSurfelShininess(float &s) { s = (float)surfelBuffer.shininess; }

    PI_INLINE void  readSurfelTangentAxisOne3(double &x, double &y, double &z)
    {
      x = surfelBuffer.tangentAxes[0][0];
      y = surfelBuffer.tangentAxes[0][1];
      z = surfelBuffer.tangentAxes[0][2];
    }

    PI_INLINE void  readSurfelTangentAxisOne3(float &x, float &y, float &z)
    {
      x = (float)surfelBuffer.tangentAxes[0][0];
      y = (float)surfelBuffer.tangentAxes[0][1];
      z = (float)surfelBuffer.tangentAxes[0][2];
    }

    PI_INLINE void  readSurfelTangentAxisTwo3(double &x, double &y, double &z)
    {
      x = surfelBuffer.tangentAxes[1][0];
      y = surfelBuffer.tangentAxes[1][1];
      z = surfelBuffer.tangentAxes[1][2];
    }

    PI_INLINE void  readSurfelTangentAxisTwo3(float &x, float &y, float &z)
    {
      x = (float)surfelBuffer.tangentAxes[1][0];
      y = (float)surfelBuffer.tangentAxes[1][1];
      z = (float)surfelBuffer.tangentAxes[1][2];
    }

    PI_INLINE uint32  readSurfelUserFlags(void) { return surfelBuffer.userFlags; }
    PI_INLINE void  readSurfelUserFlags(unsigned &flags) { flags = surfelBuffer.userFlags; }
    PI_INLINE void  readSurfelUserFlags(uint32 &flags) { flags = surfelBuffer.userFlags; }
  };

  class SFL_API OutStream : public sfl::Stream
  {
    bool  wroteHeader;
    bool  wroteSurfelSetList;

    bool  inHeader;
    bool  inSurfelSetList;
    bool  inSurfelSet;
    bool  inResolution;

    int  numAnnouncedSurfelSets;
    int  numWrittenSurfelSets;

    int  numAnnouncedResolutions;
    int  numWrittenResolutions;

    int  numAnnouncedSurfels;
    int  numWrittenSurfels;

    int  gffDepth;

    void  assertHeader(void);
    void  assertInSurfelSetList(void);

    OutStream();
    ~OutStream();

  public:
    static OutStream  *open(const char *fName, const char *mode="");
    static int         close(OutStream *out);

    int  beginHeader(void);
    OutStream  &setTitle(const char *title);
    OutStream  &setAuthor(const char *author);
    OutStream  &setApplication(const char *appl);
    OutStream  &setDescription(const char *descr);
    OutStream  &setVersion(uint32 version);
    OutStream  &setDate(const char *date);
    OutStream  &setRightVec3(double x, double y, double z);
    OutStream  &setRightVec3(const float *vec);
    OutStream  &setRightVec3(const double *vec);
    OutStream  &setUpVec3(double x, double y, double z);
    OutStream  &setUpVec3(const float *vec);
    OutStream  &setUpVec3(const double *vec);
    OutStream  &setBackVec3(double x, double y, double z);
    OutStream  &setBackVec3(const float *vec);
    OutStream  &setBackVec3(const double *vec);
    OutStream  &setDefaultUnits(double units);
    OutStream  &setOverAllBoundingBox(double x1, double y1, double z1, double x2, double y2, double z2);
    OutStream  &setSceneCameraTransform4x4(const float *mat);
    OutStream  &setSceneCameraTransform4x4(const double *mat);
    OutStream  &setSceneCameraTransformTwice4x4(const float *matSecond, const float *matFirst);
    OutStream  &setSceneCameraTransformTwice4x4(const double *matSecond, const double *matFirst);
    int  endHeader(void);

    int  beginSurfelSetList(uint32 numSurfelSets);
    // call begin/endSurfelSet() pairs inbetween
    int  endSurfelSetList(void);

    int  beginSurfelSet(unsigned propertyMask, ssize_t numResolutions=1);
    int  setSurfelSetRes0Index(int idx);
    int  setSurfelSetUnits(double units);
    int  setSurfelSetTransform3x3(const float *mat);
    int  setSurfelSetTransform4x4(const float *mat);
    int  setSurfelSetTransform3x3(const double *mat);
    int  setSurfelSetTransform4x4(const double *mat);
    int  setSurfelSetTransformTwice4x4(const float *matSecond, const float *matFirst);
    int  setSurfelSetTransformTwice4x4(const double *matSecond, const double *matFirst);
    int  setSurfelSetBoundingBox(double x1, double y1, double z1, double x2, double y2, double z2);
    // call begin/endResolution() pairs inbetween
    int  setSurfelSetIdentifier(const char *ident);
    int  setSurfelSetMarkerSurfelIndices(int n, const uint32 *idxvec);
    int  setSurfelSetMarkerSurfelUs(int n, const float *uvec); // Never write a different number of u's than indices!!
    int  setSurfelSetMarkerSurfelVs(int n, const float *vvec); // Never write a different number of v's than indices!!
    int  endSurfelSet(void);

    int  beginResolution(ssize_t numSurfels);
    // call begin/endSurfel() pairs inbetween
    int  endResolution(void);

    PI_INLINE int  beginSurfel(void) { return 0; }
    PI_INLINE int  endSurfel(void)
    {
      surfelBuffer.applyConversions();
      rwBufferHead += (surfelBuffer.*surfelBuffer.spill)(rwBufferHead);
      if (rwBufferHead >= rwBuffer[1])
        if (flushRwBuffer0() != rwBufferSize)
          return 1;
      numWrittenSurfels++;
      return 0;
    }

    PI_INLINE void  dumpSurfelPosition3(double x, double y, double z)
    {
      surfelBuffer.position[0] = x;
      surfelBuffer.position[1] = y;
      surfelBuffer.position[2] = z;
    }

    PI_INLINE void  dumpSurfelPosition4(double x, double y, double z, double w)
    {
      surfelBuffer.position[0] = x/w;
      surfelBuffer.position[1] = y/w;
      surfelBuffer.position[2] = z/w;
    }

    PI_INLINE void  dumpSurfelNormal3(double x, double y, double z)
    {
      surfelBuffer.normal[0] = x;
      surfelBuffer.normal[1] = y;
      surfelBuffer.normal[2] = z;
    }

    PI_INLINE void  dumpSurfelNormal4(double x, double y, double z, double w)
    {
      surfelBuffer.normal[0] = x/w;
      surfelBuffer.normal[1] = y/w;
      surfelBuffer.normal[2] = z/w;
    }

    PI_INLINE void  dumpSurfelRadius(double r) { surfelBuffer.radius = r; }

    PI_INLINE void  dumpSurfelWeight(double w) { surfelBuffer.weight = w; }

    PI_INLINE void  dumpSurfelSamplingDensity3(double x, double y, double z)
    {
      surfelBuffer.samplingDensity[0] = x;
      surfelBuffer.samplingDensity[1] = y;
      surfelBuffer.samplingDensity[2] = z;
    }

    PI_INLINE void  dumpSurfelSamplingDensity4(double x, double y, double z, double w)
    {
      surfelBuffer.samplingDensity[0] = x/w;
      surfelBuffer.samplingDensity[1] = y/w;
      surfelBuffer.samplingDensity[2] = z/w;
    }

    PI_INLINE void  dumpSurfelTextureUV(double u, double v)
    {
      surfelBuffer.textureUV[0] = u;
      surfelBuffer.textureUV[1] = v;
    }

    PI_INLINE void  dumpSurfelColorRf(colorEnum colIdx, float r)
    {
      surfelBuffer.colors[(int)colIdx][0] = (unsigned char)(255.0f*r + 0.5);
    }

    PI_INLINE void  dumpSurfelColorGf(colorEnum colIdx, float g)
    {
      surfelBuffer.colors[(int)colIdx][1] = (unsigned char)(255.0f*g + 0.5);
    }

    PI_INLINE void  dumpSurfelColorBf(colorEnum colIdx, float b)
    {
      surfelBuffer.colors[(int)colIdx][2] = (unsigned char)(255.0f*b + 0.5);
    }

    PI_INLINE void  dumpSurfelColorAf(colorEnum colIdx, float a)
    {
      surfelBuffer.colors[(int)colIdx][3] = (unsigned char)(255.0f*a + 0.5);
    }

    PI_INLINE void  dumpSurfelColorLf(colorEnum colIdx, float l)
    {
      surfelBuffer.colors[(int)colIdx][0] = (unsigned char)(255.0f*l + 0.5);
      surfelBuffer.colors[(int)colIdx][1] = surfelBuffer.colors[(int)colIdx][0];
      surfelBuffer.colors[(int)colIdx][2] = surfelBuffer.colors[(int)colIdx][1];
    }

    PI_INLINE void  dumpSurfelColorLAf(colorEnum colIdx, float l, float a)
    {
      surfelBuffer.colors[(int)colIdx][0] = (unsigned char)(255.0f*l + 0.5);
      surfelBuffer.colors[(int)colIdx][1] = surfelBuffer.colors[(int)colIdx][0];
      surfelBuffer.colors[(int)colIdx][2] = surfelBuffer.colors[(int)colIdx][1];
      surfelBuffer.colors[(int)colIdx][3] = (unsigned char)(255.0f*a + 0.5);
    }

    PI_INLINE void  dumpSurfelColorRGBf(colorEnum colIdx, float r, float g, float b)
    {
      surfelBuffer.colors[(int)colIdx][0] = (unsigned char)(255.0f*r + 0.5);
      surfelBuffer.colors[(int)colIdx][1] = (unsigned char)(255.0f*g + 0.5);
      surfelBuffer.colors[(int)colIdx][2] = (unsigned char)(255.0f*b + 0.5);
    }

    PI_INLINE void  dumpSurfelColorRGBAf(colorEnum colIdx, float r, float g, float b, float a)
    {
      surfelBuffer.colors[(int)colIdx][0] = (unsigned char)(255.0f*r + 0.5);
      surfelBuffer.colors[(int)colIdx][1] = (unsigned char)(255.0f*g + 0.5);
      surfelBuffer.colors[(int)colIdx][2] = (unsigned char)(255.0f*b + 0.5);
      surfelBuffer.colors[(int)colIdx][3] = (unsigned char)(255.0f*a + 0.5);
    }

#   define  dumpSurfelColorYf     dumpSurfelColorRf
#   define  dumpSurfelColorUf     dumpSurfelColorGf
#   define  dumpSurfelColorVf     dumpSurfelColorBf
#   define  dumpSurfelColorYUVf   dumpSurfelColorRGBf
#   define  dumpSurfelColorYUVAf  dumpSurfelColorRGBAf

    PI_INLINE void  dumpSurfelColorRuc(colorEnum colIdx, unsigned char r)
    {
      surfelBuffer.colors[(int)colIdx][0] = r;
    }

    PI_INLINE void  dumpSurfelColorGuc(colorEnum colIdx, unsigned char g)
    {
      surfelBuffer.colors[(int)colIdx][1] = g;
    }

    PI_INLINE void  dumpSurfelColorBuc(colorEnum colIdx, unsigned char b)
    {
      surfelBuffer.colors[(int)colIdx][2] = b;
    }

    PI_INLINE void  dumpSurfelColorAuc(colorEnum colIdx, unsigned char a)
    {
      surfelBuffer.colors[(int)colIdx][3] = a;
    }

    PI_INLINE void  dumpSurfelColorLuc(colorEnum colIdx, unsigned char l)
    {
      surfelBuffer.colors[(int)colIdx][0] = l;
      surfelBuffer.colors[(int)colIdx][1] = l;
      surfelBuffer.colors[(int)colIdx][2] = l;
    }

    PI_INLINE void  dumpSurfelColorLAuc(colorEnum colIdx, unsigned char l, unsigned char a)
    {
      surfelBuffer.colors[(int)colIdx][0] = l;
      surfelBuffer.colors[(int)colIdx][1] = l;
      surfelBuffer.colors[(int)colIdx][2] = l;
      surfelBuffer.colors[(int)colIdx][3] = a;
    }

    PI_INLINE void  dumpSurfelColorRGBuc(colorEnum colIdx, unsigned char r, unsigned char g, unsigned char b)
    {
      surfelBuffer.colors[(int)colIdx][0] = r;
      surfelBuffer.colors[(int)colIdx][1] = g;
      surfelBuffer.colors[(int)colIdx][2] = b;
    }

    PI_INLINE void  dumpSurfelColorRGBAuc(colorEnum colIdx, unsigned char r, unsigned char g, unsigned char b, unsigned char a)
    {
      surfelBuffer.colors[(int)colIdx][0] = r;
      surfelBuffer.colors[(int)colIdx][1] = g;
      surfelBuffer.colors[(int)colIdx][2] = b;
      surfelBuffer.colors[(int)colIdx][3] = a;
    }

#   define  dumpSurfelColorYuc     dumpSurfelColorRuc
#   define  dumpSurfelColorUuc     dumpSurfelColorGuc
#   define  dumpSurfelColorVuc     dumpSurfelColorBuc
#   define  dumpSurfelColorYUVuc   dumpSurfelColorRGBuc
#   define  dumpSurfelColorYUVAuc  dumpSurfelColorRGBAuc

    PI_INLINE void  dumpSurfelColorCoefff(colorEnum colIdx, float c)
    {
      // FIXME: Is that Tisu's semantic?
      surfelBuffer.coeffs[(int)colIdx] = (unsigned char)(255.0f*c + 0.5);
    }

    PI_INLINE void  dumpSurfelColorCoeffuc(colorEnum colIdx, unsigned char c)
    {
      surfelBuffer.coeffs[(int)colIdx] = c;
    }

    PI_INLINE void  dumpSurfelShininess(double s) { surfelBuffer.shininess = s; }

    PI_INLINE void  dumpSurfelTangentAxisOne3(double x, double y, double z)
    {
      surfelBuffer.tangentAxes[0][0] = x;
      surfelBuffer.tangentAxes[0][1] = y;
      surfelBuffer.tangentAxes[0][2] = z;
    }

    PI_INLINE void  dumpSurfelTangentAxisTwo3(double x, double y, double z)
    {
      surfelBuffer.tangentAxes[1][0] = x;
      surfelBuffer.tangentAxes[1][1] = y;
      surfelBuffer.tangentAxes[1][2] = z;
    }

    PI_INLINE void  dumpSurfelUserFlags(unsigned flags) { surfelBuffer.userFlags = flags; }
  };
};

#endif

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
