//
// Title:   ParameterizationDialogImpl.cpp
// Created: Mon Sep 23 14:34:37 2002
// Authors: Oliver Knoll, Mark Pauly, Matthias Zwicker
//
// Copyright (c) 2001, 2002, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlist.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qapplication.h>

#include "../../../../Core/Brush/src/Brush.h"
#include "../../../../Core/CoreResourceTools/src/CoreResourceTools.h"
#include "../../../../Core/CoreResourceTools/BitmapResourceTool/src/BitmapResourceTool.h"
#include "../../../../Core/CoreTools/src/CoreTools.h"

#include "../../../../Core/MarkerManager/src/MarkerManager.h"
#include "../UI/ExtendedSettings.h"
#include "BitmapMarker.h"
#include "ParameterizationTool.h"
#include "ParameterizationDialogImpl.h"

// ******
// global
// ******

// the single ParameterizationDialogImpl instance, only
// accesible by getInstance() and
// destroyInstance()
static ParameterizationDialogImpl *instance = 0;

// *******************
// private c'tor/d'tor
// *******************

ParameterizationDialogImpl::ParameterizationDialogImpl (ParameterizationTool *parameterizationTool,
									                    QWidget* parent,
									                    const char* name,
									                    bool modal,
									                    WFlags flags)
    : ParameterizationDialogBase (parent, name, modal, flags) {

	this->parameterizationTool  = parameterizationTool;
	bitmapMarker                = BitmapMarker::getInstance();
	currentBrush                = 0;

	extendedSettings = new ExtendedSettings (this);
	this->setOrientation (Vertical);
	this->setExtension (extendedSettings);

	this->initDialog();

}

ParameterizationDialogImpl::~ParameterizationDialogImpl() {
	BitmapMarker::destroyInstance();
}

// **************
// public methods
// **************

ParameterizationDialogImpl *ParameterizationDialogImpl::getInstance (ParameterizationTool *parameterizationTool) {

	if (instance == 0) {
		instance = new ParameterizationDialogImpl (parameterizationTool);
	}
	return instance;

}

void ParameterizationDialogImpl::destroyInstance() {

	if (instance != 0) {
		instance->close();
		delete instance;
	}
	instance = 0;

}

// ************
// public slots
// ************

void ParameterizationDialogImpl::handleRemoveSelected2D() {
	MarkerManager::getInstance()->removeSelectedFittingConstraints2D();	
}

void ParameterizationDialogImpl::handleRemoveAll2D() {
	MarkerManager::getInstance()->removeAllFittingConstraints2D();
}

void ParameterizationDialogImpl::handleRemoveSelected3D() {
	MarkerManager::getInstance()->removeSelectedFittingConstraints3D();
}

void ParameterizationDialogImpl::handleRemoveAll3D() {
	MarkerManager::getInstance()->removeAllFittingConstraints3D (true);
}

void ParameterizationDialogImpl::handleMarkPropertyBitmap() {

	this->updatePixmaps();
	bitmapMarker->show();
	bitmapMarker->raise();
	

}

void ParameterizationDialogImpl::handlePropertyChosen (const QString &newProperty) {

	currentProperty = SurfelInterface::getSystemProperty (newProperty);
	this->updatePixmaps();

}

void ParameterizationDialogImpl::handleClearSelection() {
	CoreTools::getInstance()->getSelectionTool()->clearSelection (true);
}

void ParameterizationDialogImpl::handleSelectAll() {
	CoreTools::getInstance()->getSelectionTool()->selectAllVisible (true);
}

void ParameterizationDialogImpl::handleUVGeneration() {

	float newPrecision,
		  newCoverageThreshold,
	      newDisplacementScaling;
	bool  isNumber;
	uint  newNofLevels,
		  newClusterSize;

	QApplication::setOverrideCursor (Qt::waitCursor);

	// *******************
	// read the parameters
	// *******************

	// precision

	newPrecision = extendedSettings->precisionLineEdit->text().toFloat (&isNumber);
	if (isNumber == true) {

		if (newPrecision != precision) {
			parameterizationTool->parameterization->setPrecision (newPrecision);
			precision = newPrecision;
		}

	}
	else {
		// reset the precision
		extendedSettings->precisionLineEdit->setText (QString::number (precision));		
	}

	// coverage threshold

	newCoverageThreshold = extendedSettings->coverageThresholdLineEdit->text().toFloat (&isNumber);
	if (isNumber == true) {

		if (newCoverageThreshold != coverageThreshold) {
			parameterizationTool->parameterization->setCoverageThreshold (newCoverageThreshold);
			coverageThreshold = newCoverageThreshold;
		}

	}
	else {
		// reset the precision
		extendedSettings->coverageThresholdLineEdit->setText (QString::number (coverageThreshold));		
	}

	// number of levels

	newNofLevels = extendedSettings->numberOfLevelsLineEdit->text().toUInt (&isNumber);
	if (isNumber == true) {
		
		if (newNofLevels != nofLevels) {
			parameterizationTool->parameterization->setNofLevels (newNofLevels);
			nofLevels = newNofLevels;
		}

	} else {
		extendedSettings->numberOfLevelsLineEdit->setText (QString::number (nofLevels));
	}

	// cluster size

	newClusterSize = extendedSettings->clusterSizeLineEdit->text().toUInt (&isNumber);
	if (isNumber == true) {

		if (newClusterSize != clusterSize) {
			parameterizationTool->parameterization->setClusterSize (newClusterSize);
			clusterSize = newClusterSize;
		}

	} else {
		extendedSettings->clusterSizeLineEdit->setText (QString::number (clusterSize));
	}

	// displacement scaling

	newDisplacementScaling = sculptScaleLineEdit->text().toFloat (&isNumber);
	if (isNumber == true) {

		if (newDisplacementScaling != displacementScaling) {
			parameterizationTool->parameterization->setDisplacementScaling (newDisplacementScaling);
			displacementScaling = newDisplacementScaling;
		}

	}
	else {
		// reset the displacement scaling
		sculptScaleLineEdit->setText (QString::number (displacementScaling));		
	}

	parameterizationTool->parameterization->generateUVCoordinates(samplingBitmapRadioButton->isChecked());

	QApplication::restoreOverrideCursor();

}

void ParameterizationDialogImpl::handleExtensions (bool enable) {
	this->showExtension (enable);
}

void ParameterizationDialogImpl::handleShowAlpha (bool) {
	this->updatePixmaps();
}


// ***************
// private methods
// ***************

void ParameterizationDialogImpl::initDialog() {	

	QStringList   propertiesList;
	MarkerManager *markerManager;

	propertiesList = SurfelInterface::getEditableProperties();
	propertyComboBox->insertStringList (propertiesList);
	currentProperty = SurfelInterface::getSystemProperty (propertyComboBox->currentText());
	currentBrush    = CoreResourceTools::getInstance()->getBrushChooserTool()->getSelectedBrush();

	markerManager = MarkerManager::getInstance();
	// get informed whenever a list changes
	this->connect (markerManager, SIGNAL (fittingConstraints3DChanged (const QList<Marker3D> *)),
		           this, SLOT (handleFittingConstraints3DChanged (const QList<Marker3D> *)));

	this->connect (markerManager, SIGNAL (fittingConstraints2DChanged (const QList<Marker2D> *)),
		           this, SLOT (handleFittingConstraints2DChanged (const QList<Marker2D> *)));

	
	// get informed whenever the selection changes
	this->connect (CoreTools::getInstance()->getSelectionTool(), SIGNAL (selectionChanged (SurfelInterface::Flag)),
		           this, SLOT (handleSelectionChanged (SurfelInterface::Flag)));


	// get informed whenver the brush has changed
	this->connect (CoreResourceTools::getInstance()->getBrushChooserTool(), SIGNAL (brushChanged (const Brush *)),
		           this, SLOT (handleBrushChanged (const Brush *)));

	fittingConstraint3DTextLabel->setText (QString::number (markerManager->getFittingConstraints3D()->count()));
	fittingConstraint2DTextLabel->setText (QString::number (markerManager->getFittingConstraints2D()->count()));

	precision = parameterizationTool->parameterization->getPrecision();
	extendedSettings->precisionLineEdit->setText (QString::number (precision));

	coverageThreshold = parameterizationTool->parameterization->getCoverageThreshold();
	extendedSettings->coverageThresholdLineEdit->setText (QString::number (coverageThreshold));

	nofLevels = parameterizationTool->parameterization->getNofLevels();
	extendedSettings->numberOfLevelsLineEdit->setText (QString::number (nofLevels));

	clusterSize = parameterizationTool->parameterization->getClusterSize();
	extendedSettings->clusterSizeLineEdit->setText (QString::number (clusterSize));

	displacementScaling = parameterizationTool->parameterization->getDisplacementScaling();
	sculptScaleLineEdit->setText (QString::number (displacementScaling));

	this->checkConditions();
	this->updatePixmaps();

}

void ParameterizationDialogImpl::updatePixmaps() {

	QPixmap       pixmap,
		          defaultPixmap;
	const QImage  *image;
	BrushChannel  *currentBrushChannel;


	defaultPixmap = QPixmap ("Icons/DefaultPixmap.png");
	// show property bitmap
	if (currentBrush != 0) {

		if (showAlphaCheckBox->isChecked() == false) {
			currentBrushChannel = currentBrush->getChannel (currentProperty);
		}
		else {
			currentBrushChannel = currentBrush->getAlphaChannel (currentProperty);
		}

		if (currentBrushChannel != 0) {

			image = currentBrushChannel->getBitmap();
			
			pixmap.convertFromImage (*image);
			propertyPixmapLabel->setPixmap (pixmap);

			// check and see if we also need to update the bitmap marker
			bitmapMarker->displayPixmap (&pixmap);

		}
		else {

			propertyPixmapLabel->setPixmap (defaultPixmap);
			// check and see if we also need to update the bitmap marker
			bitmapMarker->displayPixmap (&defaultPixmap);

		}

	}
	else {

		propertyPixmapLabel->setPixmap (defaultPixmap);
		bitmapMarker->displayPixmap (&defaultPixmap);

	}

}

void ParameterizationDialogImpl::checkConditions() {

	MarkerManager *markerManager;

	markerManager = MarkerManager::getInstance();
	if (CoreTools::getInstance()->getSelectionTool()->getSelectionSize() > 0 &&
		markerManager->getFittingConstraints3D()->count() > 0                               &&
		markerManager->getFittingConstraints3D()->count() == markerManager->getFittingConstraints2D()->count()) {
		generateUVButton->setEnabled (true);
	}
	else {
		generateUVButton->setEnabled (false);
	}

}


// *************
// private slots
// *************

void ParameterizationDialogImpl::handleBrushChanged (const Brush *newBrush) {

	currentBrush = newBrush;
	if (currentBrush != 0) {
		markPropertyBitmapButton->setEnabled (true);
	}
	else {
		markPropertyBitmapButton->setEnabled (false);
	}	

	this->updatePixmaps();

}

void ParameterizationDialogImpl::handleFittingConstraints3DChanged (const QList<Marker3D> *fittingConstraint3DList) {
	fittingConstraint3DTextLabel->setText (QString::number (fittingConstraint3DList->count()));
	this->checkConditions();
	qDebug ("ParameterizationDialogImpl: handleFittingConstraints3DChanged: nof 3D constraints: %d", fittingConstraint3DList->count());
}

void ParameterizationDialogImpl::handleFittingConstraints2DChanged (const QList<Marker2D> *fittingConstraint2DList) {
	fittingConstraint2DTextLabel->setText (QString::number (fittingConstraint2DList->count()));
	this->checkConditions();
}


void ParameterizationDialogImpl::handleSelectionChanged (SurfelInterface::Flag selectionFlag) {
	if (selectionFlag == SurfelInterface::SELECTED1) {
		this->checkConditions();
	}
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
